#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import netaddr

from openstack_dashboard.test.integration_tests.pages import basepage
from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables


class LaunchInstanceForm(forms.TabbedFormRegion):
    field_mappings = ((
        "availability_zone", "name", "flavor",
        "count", "source_type", "instance_snapshot_id",
        "volume_id", "volume_snapshot_id", "image_id", "volume_size",
        "vol_delete_on_instance_delete"),
        ("keypair", "groups"),
        ("script_source", "script_upload", "script_data"),
        ("disk_config", "config_drive")
    )

    def __init__(self, driver, conf):
        super(LaunchInstanceForm, self).__init__(
            driver, conf, field_mappings=self.field_mappings)


class InstancesTable(tables.TableRegion):
    name = "instances"

    @tables.bind_table_action('launch-ng')
    def launch_instance(self, launch_button):
        launch_button.click()
        return LaunchInstanceForm(self.driver, self.conf)

    @tables.bind_table_action('delete')
    def delete_instance(self, delete_button):
        delete_button.click()
        return forms.BaseFormRegion(self.driver, self.conf)


class InstancesPage(basepage.BaseNavigationPage):

    DEFAULT_FLAVOR = 'm1.tiny'
    DEFAULT_COUNT = 1
    DEFAULT_BOOT_SOURCE = 'Boot from image'
    DEFAULT_VOLUME_NAME = None
    DEFAULT_SNAPSHOT_NAME = None
    DEFAULT_VOLUME_SNAPSHOT_NAME = None
    DEFAULT_VOL_DELETE_ON_INSTANCE_DELETE = False
    DEFAULT_SECURITY_GROUP = True

    INSTANCES_TABLE_NAME_COLUMN = 'Instance Name'
    INSTANCES_TABLE_STATUS_COLUMN = 'Status'
    INSTANCES_TABLE_IP_COLUMN = 'IP Address'
    INSTANCES_TABLE_IMAGE_NAME_COLUMN = 'Image Name'

    def __init__(self, driver, conf):
        super(InstancesPage, self).__init__(driver, conf)
        self._page_title = "Instances"

    def _get_row_with_instance_name(self, name):
        return self.instances_table.get_row(self.INSTANCES_TABLE_NAME_COLUMN,
                                            name)

    def _get_rows_with_instances_names(self, names):
        return [self.instances_table.get_row(
            self.INSTANCES_TABLE_IMAGE_NAME_COLUMN, n) for n in names]

    @property
    def instances_table(self):
        return InstancesTable(self.driver, self.conf)

    def is_instance_present(self, name):
        return bool(self._get_row_with_instance_name(name))

    def create_instance(
            self, instance_name,
            available_zone=None,
            instance_count=DEFAULT_COUNT,
            flavor=DEFAULT_FLAVOR,
            boot_source=DEFAULT_BOOT_SOURCE,
            source_name=None,
            device_size=None,
            vol_delete_on_instance_delete=DEFAULT_VOL_DELETE_ON_INSTANCE_DELETE
    ):
        if not available_zone:
            available_zone = self.conf.launch_instances.available_zone
        instance_form = self.instances_table.launch_instance()
        instance_form.availability_zone.value = available_zone
        instance_form.name.text = instance_name
        instance_form.flavor.text = flavor
        instance_form.count.value = instance_count
        instance_form.source_type.text = boot_source
        boot_source = self._get_source_name(instance_form, boot_source,
                                            self.conf.launch_instances)
        if not source_name:
            source_name = boot_source[1]
        boot_source[0].text = source_name
        if device_size:
            instance_form.volume_size.value = device_size
        if vol_delete_on_instance_delete:
            instance_form.vol_delete_on_instance_delete.mark()
        instance_form.submit()

    def delete_instance(self, name):
        row = self._get_row_with_instance_name(name)
        row.mark()
        confirm_delete_instances_form = self.instances_table.delete_instance()
        confirm_delete_instances_form.submit()

    def delete_instances(self, instances_names):
        for instance_name in instances_names:
            self._get_row_with_instance_name(instance_name).mark()
        confirm_delete_instances_form = self.instances_table.delete_instance()
        confirm_delete_instances_form.submit()

    def is_instance_deleted(self, name):
        return self.instances_table.is_row_deleted(
            lambda: self._get_row_with_instance_name(name))

    def are_instances_deleted(self, instances_names):
        return self.instances_table.are_rows_deleted(
            lambda: self._get_rows_with_instances_names(instances_names))

    def is_instance_active(self, name):
        def cell_getter():
            row = self._get_row_with_instance_name(name)
            return row and row.cells[self.INSTANCES_TABLE_STATUS_COLUMN]

        status = self.instances_table.wait_cell_status(cell_getter,
                                                       ('Active', 'Error'))
        return status == 'Active'

    def _get_source_name(self, instance, boot_source,
                         conf):
        if 'image' in boot_source:
            return instance.image_id, conf.image_name
        elif boot_source == 'Boot from volume':
            return instance.volume_id, self.DEFAULT_VOLUME_NAME
        elif boot_source == 'Boot from snapshot':
            return instance.instance_snapshot_id, self.DEFAULT_SNAPSHOT_NAME
        elif 'volume snapshot (creates a new volume)' in boot_source:
            return (instance.volume_snapshot_id,
                    self.DEFAULT_VOLUME_SNAPSHOT_NAME)

    def get_image_name(self, instance_name):
        row = self._get_row_with_instance_name(instance_name)
        return row.cells[self.INSTANCES_TABLE_IMAGE_NAME_COLUMN].text

    def get_fixed_ipv4(self, name):
        row = self._get_row_with_instance_name(name)
        ips = row.cells[self.INSTANCES_TABLE_IP_COLUMN].text
        for ip in ips.split():
            if netaddr.valid_ipv4(ip):
                return ip



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import tabs
from horizon.utils import memoized
from horizon import workflows

from openstack_dashboard import api
from openstack_dashboard.api import cinder
from openstack_dashboard.usage import quotas

from openstack_dashboard.dashboards.project.volume_groups \
    import forms as vol_group_forms
from openstack_dashboard.dashboards.project.volume_groups \
    import tables as vol_group_tables
from openstack_dashboard.dashboards.project.volume_groups \
    import tabs as vol_group_tabs
from openstack_dashboard.dashboards.project.volume_groups \
    import workflows as vol_group_workflows

CGROUP_INFO_FIELDS = ("name",
                      "description")

INDEX_URL = "horizon:project:volume_groups:index"


class IndexView(tables.DataTableView):
    table_class = vol_group_tables.GroupsTable
    page_title = _("Groups")

    def get_data(self):
        try:
            groups = api.cinder.group_list_with_vol_type_names(self.request)
        except Exception:
            groups = []
            exceptions.handle(self.request,
                              _("Unable to retrieve volume groups."))
        if not groups:
            return groups
        group_snapshots = api.cinder.group_snapshot_list(self.request)
        snapshot_groups = {gs.group_id for gs in group_snapshots}
        for g in groups:
            g.has_snapshots = g.id in snapshot_groups
        return groups


class CreateView(workflows.WorkflowView):
    workflow_class = vol_group_workflows.CreateGroupWorkflow
    template_name = 'project/volume_groups/create.html'
    page_title = _("Create Volume Group")


class UpdateView(forms.ModalFormView):
    template_name = 'project/volume_groups/update.html'
    page_title = _("Edit Group")
    form_class = vol_group_forms.UpdateForm
    success_url = reverse_lazy('horizon:project:volume_groups:index')
    submit_url = "horizon:project:volume_groups:update"

    def get_initial(self):
        group = self.get_object()
        return {'group_id': self.kwargs["group_id"],
                'name': group.name,
                'description': group.description}

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context['group_id'] = self.kwargs['group_id']
        args = (self.kwargs['group_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_object(self):
        group_id = self.kwargs['group_id']
        try:
            self._object = cinder.group_get(self.request, group_id)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve group details.'),
                              redirect=reverse(INDEX_URL))
        return self._object


class RemoveVolumesView(forms.ModalFormView):
    template_name = 'project/volume_groups/remove_vols.html'
    page_title = _("Remove Volumes from Group")
    form_class = vol_group_forms.RemoveVolsForm
    success_url = reverse_lazy('horizon:project:volume_groups:index')
    submit_url = "horizon:project:volume_groups:remove_volumes"

    def get_initial(self):
        group = self.get_object()
        return {'group_id': self.kwargs["group_id"],
                'name': group.name}

    def get_context_data(self, **kwargs):
        context = super(RemoveVolumesView, self).get_context_data(**kwargs)
        context['group_id'] = self.kwargs['group_id']
        args = (self.kwargs['group_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_object(self):
        group_id = self.kwargs['group_id']
        try:
            self._object = cinder.group_get(self.request, group_id)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve group details.'),
                              redirect=reverse(INDEX_URL))
        return self._object


class DeleteView(forms.ModalFormView):
    template_name = 'project/volume_groups/delete.html'
    page_title = _("Delete Group")
    form_class = vol_group_forms.DeleteForm
    success_url = reverse_lazy('horizon:project:volume_groups:index')
    submit_url = "horizon:project:volume_groups:delete"
    submit_label = page_title

    def get_initial(self):
        group = self.get_object()
        return {'group_id': self.kwargs["group_id"],
                'name': group.name}

    def get_context_data(self, **kwargs):
        context = super(DeleteView, self).get_context_data(**kwargs)
        context['group_id'] = self.kwargs['group_id']
        args = (self.kwargs['group_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_object(self):
        group_id = self.kwargs['group_id']
        try:
            self._object = cinder.group_get(self.request, group_id)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve group details.'),
                              redirect=reverse(INDEX_URL))
        return self._object


class ManageView(workflows.WorkflowView):
    workflow_class = vol_group_workflows.UpdateGroupWorkflow

    def get_context_data(self, **kwargs):
        context = super(ManageView, self).get_context_data(**kwargs)
        context['group_id'] = self.kwargs["group_id"]
        return context

    def _get_object(self, *args, **kwargs):
        group_id = self.kwargs['group_id']
        try:
            group = cinder.group_get(self.request, group_id)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve group details.'),
                              redirect=reverse(INDEX_URL))
        return group

    def get_initial(self):
        group = self._get_object()
        return {'group_id': group.id,
                'name': group.name,
                'description': group.description,
                'vtypes': getattr(group, "volume_types")}


class CreateSnapshotView(forms.ModalFormView):
    form_class = vol_group_forms.CreateSnapshotForm
    page_title = _("Create Group Snapshot")
    template_name = 'project/volume_groups/create_snapshot.html'
    submit_label = _("Create Snapshot")
    submit_url = "horizon:project:volume_groups:create_snapshot"
    success_url = reverse_lazy('horizon:project:vg_snapshots:index')

    def get_context_data(self, **kwargs):
        context = super(CreateSnapshotView, self).get_context_data(**kwargs)
        context['group_id'] = self.kwargs['group_id']
        args = (self.kwargs['group_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        try:
            # get number of snapshots we will be creating
            search_opts = {'group_id': context['group_id']}
            volumes = api.cinder.volume_list(self.request,
                                             search_opts=search_opts)
            num_volumes = len(volumes)
            usages = quotas.tenant_quota_usages(
                self.request, targets=('snapshots', 'gigabytes'))

            if (usages['snapshots']['used'] + num_volumes >
                    usages['snapshots']['quota']):
                raise ValueError(_('Unable to create snapshots due to '
                                   'exceeding snapshot quota limit.'))
            else:
                context['numRequestedItems'] = num_volumes
                context['usages'] = usages

        except ValueError as e:
            exceptions.handle(self.request, e.message)
            return None
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve group information.'))
        return context

    def get_initial(self):
        return {'group_id': self.kwargs["group_id"]}


class CloneGroupView(forms.ModalFormView):
    form_class = vol_group_forms.CloneGroupForm
    page_title = _("Clone Group")
    template_name = 'project/volume_groups/clone_group.html'
    submit_label = _("Clone Group")
    submit_url = "horizon:project:volume_groups:clone_group"
    success_url = reverse_lazy('horizon:project:volume_groups:index')

    def get_context_data(self, **kwargs):
        context = super(CloneGroupView, self).get_context_data(**kwargs)
        context['group_id'] = self.kwargs['group_id']
        args = (self.kwargs['group_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        try:
            # get number of volumes we will be creating
            group_id = context['group_id']

            search_opts = {'group_id': group_id}
            volumes = api.cinder.volume_list(self.request,
                                             search_opts=search_opts)
            num_volumes = len(volumes)
            usages = quotas.tenant_quota_usages(
                self.request, targets=('volumes', 'gigabytes'))

            if (usages['volumes']['used'] + num_volumes >
                    usages['volumes']['quota']):
                raise ValueError(_('Unable to create group due to '
                                   'exceeding volume quota limit.'))
            else:
                context['numRequestedItems'] = num_volumes
                context['usages'] = usages

        except ValueError as e:
            exceptions.handle(self.request, e.message)
            return None
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve group information.'))
        return context

    def get_initial(self):
        return {'group_id': self.kwargs["group_id"]}


class DetailView(tabs.TabView):
    tab_group_class = vol_group_tabs.GroupsDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ group.name|default:group.id }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        group = self.get_data()
        table = vol_group_tables.GroupsTable(self.request)
        context["group"] = group
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(group)
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            group_id = self.kwargs['group_id']
            group = api.cinder.group_get_with_vol_type_names(self.request,
                                                             group_id)
            search_opts = {'group_id': group_id}
            volumes = api.cinder.volume_list(self.request,
                                             search_opts=search_opts)
            group.volume_names = [{'id': vol.id, 'name': vol.name}
                                  for vol in volumes]
            group_snapshots = api.cinder.group_snapshot_list(
                self.request, search_opts=search_opts)
            group.has_snapshots = bool(group_snapshots)
        except Exception:
            redirect = self.get_redirect_url()
            exceptions.handle(self.request,
                              _('Unable to retrieve group details.'),
                              redirect=redirect)
        return group

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:volume_groups:index')

    def get_tabs(self, request, *args, **kwargs):
        group = self.get_data()
        return self.tab_group_class(request, group=group, **kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
from importlib import import_module
import logging
import os
import pkgutil
import re

from horizon.utils import file_discovery
from openstack_dashboard import theme_settings


def import_submodules(module):
    """Import all submodules and make them available in a dict."""
    submodules = {}
    for loader, name, ispkg in pkgutil.iter_modules(module.__path__,
                                                    module.__name__ + '.'):
        try:
            submodule = import_module(name)
        except ImportError as e:
            # FIXME: Make the errors non-fatal (do we want that?).
            logging.warning("Error importing %s", name)
            logging.exception(e)
        else:
            parent, child = name.rsplit('.', 1)
            submodules[child] = submodule
    return submodules


def import_dashboard_config(modules):
    """Imports configuration from all the modules and merges it."""
    config = collections.defaultdict(dict)
    for module in modules:
        for submodule in import_submodules(module).values():
            if hasattr(submodule, 'DASHBOARD'):
                dashboard = submodule.DASHBOARD
                config[dashboard].update(submodule.__dict__)
            elif (hasattr(submodule, 'PANEL') or
                  hasattr(submodule, 'PANEL_GROUP') or
                  hasattr(submodule, 'FEATURE')):
                # If enabled and local.enabled contains a same filename,
                # the file loaded later (i.e., local.enabled) will be used.
                name = submodule.__name__.rsplit('.', 1)[1]
                config[name] = submodule.__dict__
            else:
                logging.warning("Skipping %s because it doesn't have DASHBOARD"
                                ", PANEL, PANEL_GROUP, or FEATURE defined.",
                                submodule.__name__)
    return sorted(config.items(),
                  key=lambda c: c[1]['__name__'].rsplit('.', 1)[1])


def update_dashboards(modules, horizon_config, installed_apps):
    """Imports dashboard and panel configuration from modules and applies it.

    The submodules from specified modules are imported, and the configuration
    for the specific dashboards is merged, with the later modules overriding
    settings from the former. Then the configuration is applied to
    horizon_config and installed_apps, in alphabetical order of files from
    which the configurations were imported.

    For example, given this setup:

        | foo/__init__.py
        | foo/_10_baz.py
        | foo/_20_qux.py

        | bar/__init__.py
        | bar/_30_baz_.py

    and being called with ``modules=[foo, bar]``, we will first have the
    configuration from ``_10_baz`` and ``_30_baz`` merged, then the
    configurations will be applied in order ``qux``, ``baz`` (``baz`` is
    second, because the most recent file which contributed to it, ``_30_baz``,
    comes after ``_20_qux``).

    Panel specific configurations are stored in horizon_config. Dashboards
    from both plugin-based and openstack_dashboard must be registered before
    the panel configuration can be applied. Making changes to the panel is
    deferred until the horizon autodiscover is completed, configurations are
    applied in alphabetical order of files where it was imported.
    """
    config_dashboards = horizon_config.get('dashboards', [])
    if config_dashboards or horizon_config.get('default_dashboard'):
        logging.warning(
            '"dashboards" and "default_dashboard" in (local_)settings is '
            'DEPRECATED now and may be unsupported in some future release. '
            'The preferred way to specify the order of dashboards and the '
            'default dashboard is the pluggable dashboard mechanism (in %s).',
            ', '.join([os.path.abspath(module.__path__[0])
                       for module in modules])
        )

    enabled_dashboards = []
    disabled_dashboards = []
    exceptions = horizon_config.get('exceptions', {})
    apps = []
    angular_modules = []
    js_files = []
    js_spec_files = []
    scss_files = []
    xstatic_modules = []
    panel_customization = []
    header_sections = []
    extra_tabs = collections.defaultdict(tuple)
    extra_steps = collections.defaultdict(tuple)
    update_horizon_config = {}
    for key, config in import_dashboard_config(modules):
        if config.get('DISABLED', False):
            if config.get('DASHBOARD'):
                disabled_dashboards.append(config.get('DASHBOARD'))
            continue

        _apps = config.get('ADD_INSTALLED_APPS', [])
        apps.extend(_apps)

        _header_sections = config.get('ADD_HEADER_SECTIONS', [])
        header_sections.extend(_header_sections)

        if config.get('AUTO_DISCOVER_STATIC_FILES', False):
            for _app in _apps:
                module = import_module(_app)
                base_path = os.path.join(module.__path__[0], 'static/')
                file_discovery.populate_horizon_config(horizon_config,
                                                       base_path)

        add_exceptions = config.get('ADD_EXCEPTIONS', {}).items()
        for category, exc_list in add_exceptions:
            exceptions[category] = tuple(set(exceptions.get(category, ()) +
                                             exc_list))

        angular_modules.extend(config.get('ADD_ANGULAR_MODULES', []))
        # avoid pulling in dashboard javascript dependencies multiple times
        existing = set(js_files)
        js_files.extend([f for f in config.get('ADD_JS_FILES', [])
                         if f not in existing])
        js_spec_files.extend(config.get('ADD_JS_SPEC_FILES', []))
        scss_files.extend(config.get('ADD_SCSS_FILES', []))
        xstatic_modules.extend(config.get('ADD_XSTATIC_MODULES', []))
        update_horizon_config.update(
            config.get('UPDATE_HORIZON_CONFIG', {}))
        if config.get('DASHBOARD'):
            dashboard = key
            enabled_dashboards.append(dashboard)
            if config.get('DEFAULT', False):
                horizon_config['default_dashboard'] = dashboard
        elif config.get('PANEL') or config.get('PANEL_GROUP'):
            config.pop("__builtins__", None)
            panel_customization.append(config)
        _extra_tabs = config.get('EXTRA_TABS', {})
        for tab_key, tab_defs in _extra_tabs.items():
            extra_tabs[tab_key] += tuple(tab_defs)
        _extra_steps = config.get('EXTRA_STEPS', {})
        for step_key, step_defs in _extra_steps.items():
            extra_steps[step_key] += tuple(step_defs)
    # Preserve the dashboard order specified in settings
    dashboards = ([d for d in config_dashboards
                   if d not in disabled_dashboards] +
                  [d for d in enabled_dashboards
                   if d not in config_dashboards])

    horizon_config['panel_customization'] = panel_customization
    horizon_config['header_sections'] = header_sections
    horizon_config['dashboards'] = tuple(dashboards)
    horizon_config.setdefault('exceptions', {}).update(exceptions)
    horizon_config.update(update_horizon_config)
    horizon_config.setdefault('angular_modules', []).extend(angular_modules)
    horizon_config.setdefault('js_files', []).extend(js_files)
    horizon_config.setdefault('js_spec_files', []).extend(js_spec_files)
    horizon_config.setdefault('scss_files', []).extend(scss_files)
    horizon_config.setdefault('xstatic_modules', []).extend(xstatic_modules)
    horizon_config['extra_tabs'] = extra_tabs
    horizon_config['extra_steps'] = extra_steps

    # apps contains reference to applications declared in the enabled folder
    # basically a list of applications that are internal and external plugins
    # installed_apps contains reference to applications declared in settings
    # such as django.contribe.*, django_pyscss, compressor, horizon, etc...
    # for translation, we are only interested in the list of external plugins
    # so we save the reference to it before we append to installed_apps
    horizon_config.setdefault('plugins', []).extend(apps)
    installed_apps[0:0] = apps


# Order matters, list the xstatic module name and the entry point file(s) for
# that module (this is often defined as the "main" in bower.json, and
# as the xstatic module MAIN variable in the very few compliant xstatic
# modules). If the xstatic module does define a MAIN then set the files
# list to None.
# This list is to be used as the base list which is potentially added to in
# local_settings.py before being passed to get_xstatic_dirs()
BASE_XSTATIC_MODULES = [
    ('xstatic.pkg.jquery', ['jquery.js']),
    ('xstatic.pkg.jquery_migrate', ['jquery-migrate.js']),
    ('xstatic.pkg.angular', [
        'angular.js',
        'angular-cookies.js',
        'angular-sanitize.js',
        'angular-route.js'
    ]),
    ('xstatic.pkg.angular_bootstrap', ['angular-bootstrap.js']),
    ('xstatic.pkg.angular_gettext', None),
    ('xstatic.pkg.angular_lrdragndrop', None),
    ('xstatic.pkg.angular_smart_table', None),
    ('xstatic.pkg.angular_fileupload', ['ng-file-upload-all.js']),
    ('xstatic.pkg.d3', ['d3.js']),
    ('xstatic.pkg.jquery_quicksearch', ['jquery.quicksearch.js']),
    ('xstatic.pkg.jquery_tablesorter', ['jquery.tablesorter.js']),
    ('xstatic.pkg.jquery_ui', ['jquery-ui.js']),
    ('xstatic.pkg.bootstrap_scss', ['js/bootstrap.js']),
    ('xstatic.pkg.bootstrap_datepicker', ['bootstrap-datepicker.js']),
    ('xstatic.pkg.hogan', ['hogan.js']),
    ('xstatic.pkg.rickshaw', ['rickshaw.js']),
    ('xstatic.pkg.jsencrypt', None),
    ('xstatic.pkg.objectpath', ['ObjectPath.js']),
    ('xstatic.pkg.tv4', ['tv4.js']),
    ('xstatic.pkg.angular_schema_form', ['schema-form.js']),

    # @imported in scss files diectly
    ('xstatic.pkg.font_awesome', []),
    ('xstatic.pkg.bootswatch', []),
    ('xstatic.pkg.roboto_fontface', []),
    ('xstatic.pkg.mdi', []),

    # testing only, not included in application
    ('xstatic.pkg.jasmine', []),
    ('xstatic.pkg.termjs', []),
]


def get_xstatic_dirs(XSTATIC_MODULES, HORIZON_CONFIG):
    """Discover static file configuration of the xstatic modules.

    For each entry in the XSTATIC_MODULES list we determine the entry
    point files (which may come from the xstatic MAIN var) and then
    determine where in the Django static tree the xstatic package's contents
    should be placed.

    For jquery.bootstrap.wizard.js the module name is None the static file is
    actually a 3rd-party file but resides in the Horizon source tree and not
    an xstatic package.

    The xstatic.pkg.jquery_ui package had its contents moved by packagers so
    it must be handled as a special case.
    """
    STATICFILES_DIRS = []
    HORIZON_CONFIG.setdefault('xstatic_lib_files', [])
    for module_name, files in XSTATIC_MODULES:
        module = import_module(module_name)
        if module_name == 'xstatic.pkg.jquery_ui':
            # determine the correct path for jquery-ui which packagers moved
            if module.VERSION.startswith('1.10.'):
                # The 1.10.x versions already contain 'ui' directory.
                files = ['ui/' + files[0]]

        STATICFILES_DIRS.append(
            ('horizon/lib/' + module.NAME, module.BASE_DIR)
        )

        # pull the file entry points from the xstatic package MAIN if possible
        if hasattr(module, 'MAIN'):
            files = module.MAIN
            if not isinstance(files, list):
                files = [files]

            # just the Javascript files, please (don't <script> css, etc
            # which is explicitly included in style/themes as appropriate)
            files = [file for file in files if file.endswith('.js')]

        # add to the list of files to link in the HTML
        try:
            for file in files:
                file = 'horizon/lib/' + module.NAME + '/' + file
                HORIZON_CONFIG['xstatic_lib_files'].append(file)
        except TypeError:
            raise Exception(
                '%s: Nothing to include because files to include are not '
                'defined (i.e., None) in BASE_XSTATIC_MODULES list and '
                'a corresponding XStatic module does not define MAIN list.'
                % module_name)

    return STATICFILES_DIRS


def find_static_files(
        HORIZON_CONFIG,
        AVAILABLE_THEMES,
        THEME_COLLECTION_DIR,
        ROOT_PATH):
    import horizon
    import openstack_dashboard

    os_dashboard_home_dir = openstack_dashboard.__path__[0]
    horizon_home_dir = horizon.__path__[0]

    # note the path must end in a '/' or the resultant file paths will have a
    # leading "/"
    file_discovery.populate_horizon_config(
        HORIZON_CONFIG,
        os.path.join(horizon_home_dir, 'static/')
    )

    # filter out non-angular javascript code and lib
    HORIZON_CONFIG['js_files'] = ([f for f in HORIZON_CONFIG['js_files']
                                   if not f.startswith('horizon/')])

    # note the path must end in a '/' or the resultant file paths will have a
    # leading "/"
    file_discovery.populate_horizon_config(
        HORIZON_CONFIG,
        os.path.join(os_dashboard_home_dir, 'static/'),
        sub_path='app/'
    )

    # Discover theme static resources, and in particular any
    # static HTML (client-side) that the theme overrides
    theme_static_files = {}
    theme_info = theme_settings.get_theme_static_dirs(
        AVAILABLE_THEMES,
        THEME_COLLECTION_DIR,
        ROOT_PATH)

    for url, path in theme_info:
        discovered_files = {}

        # discover static files provided by the theme
        file_discovery.populate_horizon_config(
            discovered_files,
            path
        )

        # Get the theme name from the theme url
        theme_name = url.split('/')[-1]

        # build a dictionary of this theme's static HTML templates.
        # For each overridden template, strip off the '/templates/' part of the
        # theme filename then use that name as the key, and the location in the
        # theme directory as the value. This allows the quick lookup of
        # theme path for any file overridden by a theme template
        template_overrides = {}
        for theme_file in discovered_files['external_templates']:
            # Example:
            #   external_templates_dict[
            #       'framework/widgets/help-panel/help-panel.html'
            #   ] = 'themes/material/templates/framework/widgets/\
            #        help-panel/help-panel.html'
            override_path = re.sub(r'^(|.*/)templates/', '', theme_file)
            template_overrides[override_path] = os.path.join('themes',
                                                             theme_name,
                                                             theme_file)

        discovered_files['template_overrides'] = template_overrides

        # Save all of the discovered file info for this theme in our
        # 'theme_files' object using the theme name as the key
        theme_static_files[theme_name] = discovered_files

    # Add the theme file info to the horizon config for use by template tags
    HORIZON_CONFIG['theme_static_files'] = theme_static_files



# Copyright 2015, Thales Services SAS
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from neutronclient.common import exceptions as neutron_exc

from horizon import exceptions
from horizon import forms
from horizon import messages

from openstack_dashboard.api import neutron as api

LOG = logging.getLogger(__name__)


class AddRouterRoute(forms.SelfHandlingForm):
    destination = forms.IPField(
        version=forms.IPv4 | forms.IPv6,
        label=_("Destination CIDR"),
        mask=True)
    nexthop = forms.IPField(
        version=forms.IPv4 | forms.IPv6,
        label=_("Next Hop"))
    failure_url = 'horizon:project:routers:detail'

    def handle(self, request, data, **kwargs):
        router_id = self.initial['router_id']
        try:
            route = {'nexthop': data['nexthop'],
                     'destination': data['destination']}
            api.router_static_route_add(request,
                                        router_id,
                                        route)
            msg = _('Static route added')
            messages.success(request, msg)
            return True
        except neutron_exc.BadRequest as e:
            LOG.info('Invalid format for routes %(route)s: %(exc)s',
                     {'route': route, 'exc': e})
            msg = _('Invalid format for routes: %s') % e
            messages.error(request, msg)
            redirect = reverse(self.failure_url, args=[router_id])
            exceptions.handle(request, msg, redirect=redirect)
        except Exception as e:
            LOG.info('Failed to add route: %s', e)
            msg = _('Failed to add route: %s') % e
            messages.error(request, msg)
            redirect = reverse(self.failure_url, args=[router_id])
            exceptions.handle(request, msg, redirect=redirect)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse

from openstack_dashboard import api
from openstack_dashboard.test import helpers as test


class EvacuateHostViewTest(test.BaseAdminViewTests):
    @test.create_mocks({api.nova: ['service_list']})
    def test_index(self):
        hypervisor = self.hypervisors.list().pop().hypervisor_hostname
        services = [service for service in self.services.list()
                    if service.binary == 'nova-compute']
        self.mock_service_list.return_value = services

        url = reverse('horizon:admin:hypervisors:compute:evacuate_host',
                      args=[hypervisor])
        res = self.client.get(url)
        self.assertTemplateUsed(res,
                                'admin/hypervisors/compute/evacuate_host.html')

        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(), binary='nova-compute')

    @test.create_mocks({api.nova: ['service_list',
                                   'evacuate_host']})
    def test_successful_post(self):
        hypervisor = self.hypervisors.list().pop().hypervisor_hostname
        services = [service for service in self.services.list()
                    if service.binary == 'nova-compute']

        self.mock_service_list.return_value = services
        self.mock_evacuate_host.return_value = True

        url = reverse('horizon:admin:hypervisors:compute:evacuate_host',
                      args=[hypervisor])

        form_data = {'current_host': services[1].host,
                     'target_host': services[0].host,
                     'on_shared_storage': False}

        res = self.client.post(url, form_data)
        dest_url = reverse('horizon:admin:hypervisors:index')
        self.assertNoFormErrors(res)
        self.assertMessageCount(success=1)
        self.assertRedirectsNoFollow(res, dest_url)

        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(), binary='nova-compute')
        self.mock_evacuate_host.assert_called_once_with(
            test.IsHttpRequest(),
            services[1].host,
            services[0].host,
            False)

    @test.create_mocks({api.nova: ['service_list',
                                   'evacuate_host']})
    def test_failing_nova_call_post(self):
        hypervisor = self.hypervisors.list().pop().hypervisor_hostname
        services = [service for service in self.services.list()
                    if service.binary == 'nova-compute']

        self.mock_service_list.return_value = services
        self.mock_evacuate_host.side_effect = self.exceptions.nova

        url = reverse('horizon:admin:hypervisors:compute:evacuate_host',
                      args=[hypervisor])

        form_data = {'current_host': services[1].host,
                     'target_host': services[0].host,
                     'on_shared_storage': False}

        res = self.client.post(url, form_data)
        dest_url = reverse('horizon:admin:hypervisors:index')
        self.assertMessageCount(error=1)
        self.assertRedirectsNoFollow(res, dest_url)

        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(), binary='nova-compute')
        self.mock_evacuate_host.assert_called_once_with(
            test.IsHttpRequest(),
            services[1].host,
            services[0].host,
            False)


class MigrateHostViewTest(test.BaseAdminViewTests):
    def test_index(self):
        disabled_services = [service for service in self.services.list()
                             if (service.binary == 'nova-compute' and
                                 service.status == 'disabled')]
        disabled_service = disabled_services[0]

        url = reverse('horizon:admin:hypervisors:compute:migrate_host',
                      args=[disabled_service.host])
        res = self.client.get(url)
        self.assertNoMessages()
        self.assertTemplateUsed(res,
                                'admin/hypervisors/compute/migrate_host.html')

    @test.create_mocks({api.nova: ['migrate_host']})
    def test_maintenance_host_cold_migration_succeed(self):
        disabled_services = [service for service in self.services.list()
                             if (service.binary == 'nova-compute' and
                                 service.status == 'disabled')]
        disabled_service = disabled_services[0]
        self.mock_migrate_host.return_value = True

        url = reverse('horizon:admin:hypervisors:compute:migrate_host',
                      args=[disabled_service.host])
        form_data = {'current_host': disabled_service.host,
                     'migrate_type': 'cold_migrate',
                     'disk_over_commit': False,
                     'block_migration': False}
        res = self.client.post(url, form_data)
        dest_url = reverse('horizon:admin:hypervisors:index')
        self.assertNoFormErrors(res)
        self.assertMessageCount(success=1)
        self.assertRedirectsNoFollow(res, dest_url)

        self.mock_migrate_host.assert_called_once_with(
            test.IsHttpRequest(),
            disabled_service.host,
            live_migrate=False,
            disk_over_commit=False,
            block_migration=False)

    @test.create_mocks({api.nova: ['migrate_host']})
    def test_maintenance_host_live_migration_succeed(self):
        disabled_services = [service for service in self.services.list()
                             if (service.binary == 'nova-compute' and
                                 service.status == 'disabled')]
        disabled_service = disabled_services[0]
        self.mock_migrate_host.return_value = True
        url = reverse('horizon:admin:hypervisors:compute:migrate_host',
                      args=[disabled_service.host])
        form_data = {'current_host': disabled_service.host,
                     'migrate_type': 'live_migrate',
                     'disk_over_commit': False,
                     'block_migration': True}
        res = self.client.post(url, form_data)
        dest_url = reverse('horizon:admin:hypervisors:index')
        self.assertNoFormErrors(res)
        self.assertMessageCount(success=1)
        self.assertRedirectsNoFollow(res, dest_url)

        self.mock_migrate_host.assert_called_once_with(
            test.IsHttpRequest(),
            disabled_service.host,
            live_migrate=True,
            disk_over_commit=False,
            block_migration=True)

    @test.create_mocks({api.nova: ['migrate_host']})
    def test_maintenance_host_migration_fails(self):
        disabled_services = [service for service in self.services.list()
                             if (service.binary == 'nova-compute' and
                                 service.status == 'disabled')]
        disabled_service = disabled_services[0]

        self.mock_migrate_host.side_effect = self.exceptions.nova

        url = reverse('horizon:admin:hypervisors:compute:migrate_host',
                      args=[disabled_service.host])
        form_data = {'current_host': disabled_service.host,
                     'migrate_type': 'live_migrate',
                     'disk_over_commit': False,
                     'block_migration': True}
        res = self.client.post(url, form_data)
        dest_url = reverse('horizon:admin:hypervisors:index')
        self.assertMessageCount(error=1)
        self.assertRedirectsNoFollow(res, dest_url)

        self.mock_migrate_host.assert_called_once_with(
            test.IsHttpRequest(),
            disabled_service.host,
            live_migrate=True,
            disk_over_commit=False,
            block_migration=True)


class DisableServiceViewTest(test.BaseAdminViewTests):
    def test_index(self):
        hypervisor = self.hypervisors.list().pop().hypervisor_hostname

        url = reverse('horizon:admin:hypervisors:compute:disable_service',
                      args=[hypervisor])
        res = self.client.get(url)
        template = 'admin/hypervisors/compute/disable_service.html'
        self.assertTemplateUsed(res, template)

    @test.create_mocks({api.nova: ['service_disable']})
    def test_successful_post(self):
        hypervisor = self.hypervisors.list().pop().hypervisor_hostname
        services = [service for service in self.services.list()
                    if service.binary == 'nova-compute']

        self.mock_service_disable.return_value = True

        url = reverse('horizon:admin:hypervisors:compute:disable_service',
                      args=[hypervisor])

        form_data = {'host': services[0].host,
                     'reason': 'test disable'}

        res = self.client.post(url, form_data)
        dest_url = reverse('horizon:admin:hypervisors:index')
        self.assertNoFormErrors(res)
        self.assertMessageCount(success=1)
        self.assertRedirectsNoFollow(res, dest_url)

        self.mock_service_disable.assert_called_once_with(
            test.IsHttpRequest(),
            services[0].host,
            'nova-compute',
            reason='test disable')

    @test.create_mocks({api.nova: ['service_disable']})
    def test_failing_nova_call_post(self):
        hypervisor = self.hypervisors.list().pop().hypervisor_hostname
        services = [service for service in self.services.list()
                    if service.binary == 'nova-compute']

        self.mock_service_disable.side_effect = self.exceptions.nova

        url = reverse('horizon:admin:hypervisors:compute:disable_service',
                      args=[hypervisor])

        form_data = {'host': services[0].host,
                     'reason': 'test disable'}

        res = self.client.post(url, form_data)
        dest_url = reverse('horizon:admin:hypervisors:index')
        self.assertMessageCount(error=1)
        self.assertRedirectsNoFollow(res, dest_url)

        self.mock_service_disable.assert_called_once_with(
            test.IsHttpRequest(),
            services[0].host,
            'nova-compute',
            reason='test disable')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

from openstack_dashboard.test.integration_tests import decorators
from openstack_dashboard.test.integration_tests import helpers
from openstack_dashboard.test.integration_tests.regions import messages


class TestSecuritygroup(helpers.TestCase):
    SEC_GROUP_NAME = helpers.gen_random_resource_name("securitygroup")
    RULE_PORT = str(random.randint(9000, 9999))

    @property
    def securitygroup_page(self):
        return self.home_pg.\
            go_to_project_network_securitygroupspage()

    def _create_securitygroup(self):
        page = self.securitygroup_page
        page.create_securitygroup(self.SEC_GROUP_NAME)
        self.assertTrue(page.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(page.find_message_and_dismiss(messages.ERROR))
        self.assertTrue(page.is_securitygroup_present(self.SEC_GROUP_NAME))

    def _delete_securitygroup(self):
        page = self.securitygroup_page
        page.delete_securitygroup(self.SEC_GROUP_NAME)
        self.assertTrue(page.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(page.find_message_and_dismiss(messages.ERROR))
        self.assertFalse(page.is_securitygroup_present(self.SEC_GROUP_NAME))

    def _add_rule(self):
        page = self.securitygroup_page
        page = page.go_to_manage_rules(self.SEC_GROUP_NAME)
        page.create_rule(self.RULE_PORT)
        self.assertTrue(page.find_message_and_dismiss(messages.SUCCESS))
        self.assertTrue(page.is_port_present(self.RULE_PORT))

    def _delete_rule_by_table_action(self):
        page = self.securitygroup_page
        page = page.go_to_manage_rules(self.SEC_GROUP_NAME)
        page.delete_rules(self.RULE_PORT)
        self.assertTrue(page.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(page.find_message_and_dismiss(messages.ERROR))
        self.assertFalse(page.is_port_present(self.RULE_PORT))

    def _delete_rule_by_row_action(self):
        page = self.securitygroup_page
        page = page.go_to_manage_rules(self.SEC_GROUP_NAME)
        page.delete_rule(self.RULE_PORT)
        self.assertTrue(page.find_message_and_dismiss(messages.SUCCESS))
        self.assertFalse(page.find_message_and_dismiss(messages.ERROR))
        self.assertFalse(page.is_port_present(self.RULE_PORT))

    @decorators.skip_because(bugs=['1792028'])
    def test_securitygroup_create_delete(self):
        """tests the security group creation and deletion functionalities:

        * creates a new security group
        * verifies the security group appears in the security groups table
        * deletes the newly created security group
        * verifies the security group does not appear in the table after
          deletion
        """
        self._create_securitygroup()
        self._delete_securitygroup()

    @decorators.skip_because(bugs=['1792028'])
    def test_managerules_create_delete_by_row(self):
        """tests the manage rules creation and deletion functionalities:

        * create a new security group
        * verifies the security group appears in the security groups table
        * creates a new rule
        * verifies the rule appears in the rules table
        * delete the newly created rule
        * verifies the rule does not appear in the table after deletion
        * deletes the newly created security group
        * verifies the security group does not appear in the table after
          deletion
        """
        self._create_securitygroup()
        self._add_rule()
        self._delete_rule_by_row_action()
        self._delete_securitygroup()

    @decorators.skip_because(bugs=['1792028'])
    def test_managerules_create_delete_by_table(self):
        """tests the manage rules creation and deletion functionalities:

        * create a new security group
        * verifies the security group appears in the security groups table
        * creates a new rule
        * verifies the rule appears in the rules table
        * delete the newly created rule
        * verifies the rule does not appear in the table after deletion
        * deletes the newly created security group
        * verifies the security group does not appear in the table after
          deletion
        """
        self._create_securitygroup()
        self._add_rule()
        self._delete_rule_by_table_action()
        self._delete_securitygroup()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from horizon.test import helpers as test
from horizon.utils import units


class UnitsTests(test.TestCase):
    def test_is_supported(self):
        self.assertTrue(units.is_supported('MB'))
        self.assertTrue(units.is_supported('min'))
        self.assertFalse(units.is_supported('KWh'))
        self.assertFalse(units.is_supported('unknown_unit'))

    def test_is_larger(self):
        self.assertTrue(units.is_larger('KB', 'B'))
        self.assertTrue(units.is_larger('MB', 'B'))
        self.assertTrue(units.is_larger('GB', 'B'))
        self.assertTrue(units.is_larger('TB', 'B'))
        self.assertTrue(units.is_larger('GB', 'MB'))
        self.assertFalse(units.is_larger('B', 'KB'))
        self.assertFalse(units.is_larger('MB', 'GB'))

        self.assertTrue(units.is_larger('min', 's'))
        self.assertTrue(units.is_larger('hr', 'min'))
        self.assertTrue(units.is_larger('hr', 's'))
        self.assertFalse(units.is_larger('s', 'min'))

    def test_convert(self):
        self.assertEqual(units.convert(4096, 'MB', 'GB'), (4, 'GB'))
        self.assertEqual(units.convert(4, 'GB', 'MB'), (4096, 'MB'))

        self.assertEqual(units.convert(1.5, 'hr', 'min'), (90, 'min'))
        self.assertEqual(units.convert(12, 'hr', 'day'), (0.5, 'day'))

    def test_normalize(self):
        self.assertEqual(units.normalize(1, 'B'), (1, 'B'))
        self.assertEqual(units.normalize(1000, 'B'), (1000, 'B'))
        self.assertEqual(units.normalize(1024, 'B'), (1, 'KB'))
        self.assertEqual(units.normalize(1024 * 1024, 'B'), (1, 'MB'))
        self.assertEqual(units.normalize(10 * 1024 ** 3, 'B'), (10, 'GB'))
        self.assertEqual(units.normalize(1000 * 1024 ** 4, 'B'), (1000, 'TB'))
        self.assertEqual(units.normalize(1024, 'KB'), (1, 'MB'))
        self.assertEqual(units.normalize(1024 ** 2, 'KB'), (1, 'GB'))
        self.assertEqual(units.normalize(10 * 1024, 'MB'), (10, 'GB'))
        self.assertEqual(units.normalize(0.5, 'KB'), (512, 'B'))
        self.assertEqual(units.normalize(0.0001, 'MB'), (104.9, 'B'))

        self.assertEqual(units.normalize(1, 's'), (1, 's'))
        self.assertEqual(units.normalize(120, 's'), (2, 'min'))
        self.assertEqual(units.normalize(3600, 's'), (60, 'min'))
        self.assertEqual(units.normalize(3600 * 24, 's'), (24, 'hr'))
        self.assertEqual(units.normalize(10 * 3600 * 24, 's'), (10, 'day'))
        self.assertEqual(units.normalize(90, 'min'), (90, 'min'))
        self.assertEqual(units.normalize(150, 'min'), (2.5, 'hr'))
        self.assertEqual(units.normalize(60 * 24, 'min'), (24, 'hr'))
        self.assertEqual(units.normalize(0.5, 'day'), (12, 'hr'))
        self.assertEqual(units.normalize(10800000000000, 'ns'), (3, 'hr'))
        self.assertEqual(units.normalize(14, 'day'), (2, 'week'))
        self.assertEqual(units.normalize(91, 'day'), (3, 'month'))
        self.assertEqual(units.normalize(18, 'month'), (18, 'month'))
        self.assertEqual(units.normalize(24, 'month'), (2, 'year'))

        self.assertEqual(units.normalize(1, 'unknown_unit'),
                         (1, 'unknown_unit'))



# Copyright 2016, Rackspace, US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import django.utils.html


def escape(text, existing=django.utils.html.escape):
    # Replace our angular markup string with a different string
    # (which just happens to be the Django comment string)
    # this prevents user-supplied data from being intepreted in
    # our pages by angularjs, thus preventing it from being used
    # for XSS attacks. Note that we use {$ $} instead of the
    # standard {{ }} - this is configured in horizon.framework
    # angularjs module through $interpolateProvider.
    return existing(text).replace('{$', '{%').replace('$}', '%}')


# this will be invoked as early as possible in settings.py
def monkeypatch_escape():
    django.utils.html.escape = escape



# Copyright 2012 NEC Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import django.test
from django.urls import reverse

import mock
from oslo_serialization import jsonutils

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.network_topology import views
from openstack_dashboard.test import helpers as test
from openstack_dashboard.usage import quotas

JSON_URL = reverse('horizon:project:network_topology:json')
INDEX_URL = reverse('horizon:project:network_topology:index')


class NetworkTopologyTests(test.TestCase):
    trans = views.TranslationHelper()

    @test.create_mocks({api.nova: ('server_list',),
                        api.neutron: ('network_list_for_tenant',
                                      'network_list',
                                      'router_list',
                                      'port_list')})
    def test_json_view(self):
        self._test_json_view()

    @django.test.utils.override_settings(
        OPENSTACK_NEUTRON_NETWORK={'enable_router': False})
    @test.create_mocks({api.nova: ('server_list',),
                        api.neutron: ('network_list_for_tenant',
                                      'port_list')})
    def test_json_view_router_disabled(self):
        self._test_json_view(router_enable=False)

    @django.test.utils.override_settings(CONSOLE_TYPE=None)
    @test.create_mocks({api.nova: ('server_list',),
                        api.neutron: ('network_list_for_tenant',
                                      'network_list',
                                      'router_list',
                                      'port_list')})
    def test_json_view_console_disabled(self):
        self._test_json_view(with_console=False)

    def _test_json_view(self, router_enable=True, with_console=True):
        self.mock_server_list.return_value = [self.servers.list(), False]

        tenant_networks = [net for net in self.networks.list()
                           if not net['router:external']]
        external_networks = [net for net in self.networks.list()
                             if net['router:external']]
        self.mock_network_list_for_tenant.return_value = tenant_networks

        # router1 : gateway port not in the port list
        # router2 : no gateway port
        # router3 : gateway port included in port list
        routers = self.routers.list() + self.routers_with_rules.list()
        if router_enable:
            self.mock_router_list.return_value = routers
            self.mock_network_list.return_value = external_networks
        self.mock_port_list.return_value = self.ports.list()

        res = self.client.get(JSON_URL)

        self.assertEqual('text/json', res['Content-Type'])
        data = jsonutils.loads(res.content)

        # servers
        expect_server_urls = []
        for server in self.servers.list():
            expect_server = {
                'id': server.id,
                'name': server.name,
                'status': server.status.title(),
                'original_status': server.status,
                'task': None,
                'url': '/project/instances/%s/' % server.id
            }
            if server.status != 'BUILD' and with_console:
                expect_server['console'] = 'auto_console'
            expect_server_urls.append(expect_server)
        self.assertEqual(expect_server_urls, data['servers'])

        # routers
        if router_enable:
            expect_router_urls = [
                {'id': router.id,
                 'external_gateway_info':
                 router.external_gateway_info,
                 'name': router.name,
                 'status': router.status.title(),
                 'original_status': router.status,
                 'url': '/project/routers/%s/' % router.id}
                for router in routers]
            self.assertEqual(expect_router_urls, data['routers'])
        else:
            self.assertFalse(data['routers'])

        # networks
        expect_net_urls = []
        if router_enable:
            expect_net_urls += [{
                'id': net.id,
                'url': '/project/networks/%s/detail' % net.id,
                'name': net.name,
                'router:external': net.router__external,
                'status': net.status.title(),
                'original_status': net.status,
                'subnets': [{
                    'cidr': snet.cidr,
                    'id': snet.id,
                    'url': '/project/networks/subnets/%s/detail' % snet.id}
                    for snet in net.subnets]}
                for net in external_networks]
        expect_net_urls.extend([{
            'id': net.id,
            'url': '/project/networks/%s/detail' % net.id,
            'name': net.name,
            'router:external': net.router__external,
            'status': net.status.title(),
            'allow_delete_subnet': True,
            'original_status': net.status,
            'subnets': [{
                'cidr': subnet.cidr,
                'id': subnet.id,
                'url': '/project/networks/subnets/%s/detail' % subnet.id}
                for subnet in net.subnets]}
            for net in tenant_networks])
        for exp_net in expect_net_urls:
            if exp_net['url'] is None:
                del exp_net['url']
        self.assertEqual(expect_net_urls, data['networks'])

        valid_network_ids = [net.id for net in tenant_networks]
        if router_enable:
            valid_network_ids = [net.id for net in self.networks.list()]
        # ports
        expect_port_urls = [
            {'id': port.id,
             'device_id': port.device_id,
             'device_owner': port.device_owner,
             'fixed_ips': port.fixed_ips,
             'network_id': port.network_id,
             'status': port.status.title(),
             'original_status': port.status,
             'url': '/project/networks/ports/%s/detail' % port.id}
            for port in self.ports.list()
            if port.network_id in valid_network_ids]
        if router_enable:
            # fake port for router1 gateway (router1 on ext_net)
            router1 = routers[0]
            ext_net = external_networks[0]
            expect_port_urls.append(
                {'id': 'gateway%s' % ext_net.id,
                 'device_id': router1.id,
                 'network_id': ext_net.id,
                 'fixed_ips': []})
        self.assertEqual(expect_port_urls, data['ports'])

        self.mock_server_list.assert_called_once_with(
            test.IsHttpRequest())
        self.mock_network_list_for_tenant.assert_called_once_with(
            test.IsHttpRequest(), self.tenant.id,
            include_pre_auto_allocate=False)
        if router_enable:
            self.mock_router_list.assert_called_once_with(
                test.IsHttpRequest(), tenant_id=self.tenant.id)
            self.mock_network_list.assert_called_once_with(
                test.IsHttpRequest(), **{'router:external': True})
        self.mock_port_list.assert_called_once_with(
            test.IsHttpRequest())


class NetworkTopologyCreateTests(test.TestCase):

    def _test_new_button_disabled_when_quota_exceeded(
            self, expected_string,
            networks_quota=10, routers_quota=10, instances_quota=10):
        quota_data = self.quota_usages.first()
        quota_data['network']['available'] = networks_quota
        quota_data['router']['available'] = routers_quota
        quota_data['instances']['available'] = instances_quota

        self.mock_tenant_quota_usages.return_value = quota_data

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'project/network_topology/index.html')

        self.assertContains(res, expected_string, html=True,
                            msg_prefix="The create button is not disabled")

        self.mock_tenant_quota_usages.assert_has_calls([
            mock.call(test.IsHttpRequest(), targets=('instances', )),
            mock.call(test.IsHttpRequest(), targets=('network', )),
            mock.call(test.IsHttpRequest(), targets=('router', )),
        ] * 3)

    @test.create_mocks({quotas: ('tenant_quota_usages',)})
    def test_create_network_button_disabled_when_quota_exceeded(self):
        url = reverse('horizon:project:network_topology:createnetwork')
        classes = 'btn btn-default ajax-modal'
        link_name = "Create Network (Quota exceeded)"
        expected_string = "<a href='%s' class='%s disabled' "\
            "id='networks__action_create'>" \
            "<span class='fa fa-plus'></span>%s</a>" \
            % (url, classes, link_name)

        self._test_new_button_disabled_when_quota_exceeded(expected_string,
                                                           networks_quota=0)

    @test.create_mocks({quotas: ('tenant_quota_usages',)})
    def test_create_router_button_disabled_when_quota_exceeded(self):
        url = reverse('horizon:project:network_topology:createrouter')
        classes = 'btn btn-default ajax-modal'
        link_name = "Create Router (Quota exceeded)"
        expected_string = "<a href='%s' class='%s disabled' "\
            "id='Routers__action_create'>" \
            "<span class='fa fa-plus'></span>%s</a>" \
            % (url, classes, link_name)

        self._test_new_button_disabled_when_quota_exceeded(expected_string,
                                                           routers_quota=0)

    @test.update_settings(LAUNCH_INSTANCE_LEGACY_ENABLED=True)
    @test.create_mocks({quotas: ('tenant_quota_usages',)})
    def test_launch_instance_button_disabled_when_quota_exceeded(self):
        url = reverse('horizon:project:network_topology:launchinstance')
        classes = 'btn btn-default btn-launch ajax-modal'
        link_name = "Launch Instance (Quota exceeded)"
        expected_string = "<a href='%s' class='%s disabled' "\
            "id='instances__action_launch'>" \
            "<span class='fa fa-cloud-upload'></span>%s</a>" \
            % (url, classes, link_name)

        self._test_new_button_disabled_when_quota_exceeded(expected_string,
                                                           instances_quota=0)



# Copyright 2019 NEC Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
import mock

from openstack_dashboard import api

from openstack_dashboard.test import helpers as test


INDEX_URL = reverse('horizon:admin:vg_snapshots:index')
INDEX_TEMPLATE = 'horizon/common/_data_table_view.html'


class AdminGroupSnapshotTests(test.BaseAdminViewTests):
    @test.create_mocks({
        api.keystone: ['tenant_list'],
        api.cinder: ['group_list',
                     'group_snapshot_list']})
    def test_index(self):
        vg_snapshots = self.cinder_group_snapshots.list()
        groups = self.cinder_groups.list()
        tenants = self.tenants.list()
        self.mock_group_snapshot_list.return_value = vg_snapshots
        self.mock_group_list.return_value = groups
        self.mock_tenant_list.return_value = [tenants, False]

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.assertIn('volume_vg_snapshots_table', res.context)
        volume_vg_snapshots_table = res.context['volume_vg_snapshots_table']
        volume_vg_snapshots = volume_vg_snapshots_table.data
        self.assertEqual(len(volume_vg_snapshots), 1)

        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        self.mock_group_snapshot_list.assert_called_once_with(
            test.IsHttpRequest(), {'all_tenants': 1})
        self.mock_group_list.assert_called_once_with(
            test.IsHttpRequest(), {'all_tenants': 1})

    @test.create_mocks({
        api.keystone: ['tenant_list'],
        api.cinder: ['group_list',
                     'group_snapshot_delete',
                     'group_snapshot_list']})
    def test_delete_group_snapshot(self):
        vg_snapshots = self.cinder_group_snapshots.list()
        vg_snapshot = self.cinder_group_snapshots.first()
        tenants = self.tenants.list()
        self.mock_group_snapshot_list.return_value = vg_snapshots
        self.mock_group_snapshot_delete.return_value = None
        self.mock_group_list.return_value = self.cinder_groups.list()
        self.mock_tenant_list.return_value = [tenants, False]

        form_data = {'action': 'volume_vg_snapshots__delete_vg_snapshot__%s'
                     % vg_snapshot.id}
        res = self.client.post(INDEX_URL, form_data, follow=True)
        self.assertEqual(res.status_code, 200)
        self.assertIn("Scheduled deletion of Snapshot: %s" % vg_snapshot.name,
                      [m.message for m in res.context['messages']])

        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_group_snapshot_list, 2,
            mock.call(test.IsHttpRequest(), {'all_tenants': 1}))
        self.mock_group_snapshot_delete.assert_called_once_with(
            test.IsHttpRequest(), vg_snapshot.id)
        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_group_list, 2,
            mock.call(test.IsHttpRequest(), {'all_tenants': 1}))

    @test.create_mocks({
        api.keystone: ['tenant_list'],
        api.cinder: ['group_list',
                     'group_snapshot_delete',
                     'group_snapshot_list']})
    def test_delete_group_snapshot_exception(self):
        vg_snapshots = self.cinder_group_snapshots.list()
        vg_snapshot = self.cinder_group_snapshots.first()
        tenants = self.tenants.list()
        self.mock_group_snapshot_list.return_value = vg_snapshots
        self.mock_group_snapshot_delete.side_effect = self.exceptions.cinder
        self.mock_group_list.return_value = self.cinder_groups.list()
        self.mock_tenant_list.return_value = [tenants, False]

        form_data = {'action': 'volume_vg_snapshots__delete_vg_snapshot__%s'
                     % vg_snapshot.id}
        res = self.client.post(INDEX_URL, form_data, follow=True)
        self.assertEqual(res.status_code, 200)
        self.assertIn("Unable to delete snapshot: %s" % vg_snapshot.name,
                      [m.message for m in res.context['messages']])

        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_group_snapshot_list, 2,
            mock.call(test.IsHttpRequest(), {'all_tenants': 1}))
        self.mock_group_snapshot_delete.assert_called_once_with(
            test.IsHttpRequest(), vg_snapshot.id)
        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_group_list, 2,
            mock.call(test.IsHttpRequest(), {'all_tenants': 1}))

    @test.create_mocks({
        api.cinder: ['group_snapshot_get',
                     'group_get',
                     'volume_type_get',
                     'volume_list']})
    def test_detail_view(self):
        vg_snapshot = self.cinder_group_snapshots.first()
        group = self.cinder_groups.first()
        volume_type = self.cinder_volume_types.first()
        volumes = self.cinder_volumes.list()

        self.mock_group_snapshot_get.return_value = vg_snapshot
        self.mock_group_get.return_value = group
        self.mock_volume_type_get.return_value = volume_type
        self.mock_volume_list.return_value = volumes

        url = reverse(
            'horizon:admin:vg_snapshots:detail',
            args=[vg_snapshot.id])
        res = self.client.get(url)
        self.assertNoFormErrors(res)
        self.assertEqual(res.status_code, 200)
        self.mock_group_snapshot_get.assert_called_once_with(
            test.IsHttpRequest(), vg_snapshot.id)
        self.mock_group_get.assert_called_once_with(
            test.IsHttpRequest(), group.id)
        self.mock_volume_type_get.assert_called_once_with(
            test.IsHttpRequest(), volume_type.id)
        search_opts = {'group_id': group.id}
        self.mock_volume_list.assert_called_once_with(
            test.IsHttpRequest(), search_opts=search_opts)

    @test.create_mocks({api.cinder: ['group_snapshot_get']})
    def test_detail_view_with_exception(self):
        vg_snapshot = self.cinder_group_snapshots.first()

        self.mock_group_snapshot_get.side_effect = self.exceptions.cinder

        url = reverse(
            'horizon:admin:vg_snapshots:detail',
            args=[vg_snapshot.id])
        res = self.client.get(url)
        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        self.mock_group_snapshot_get.assert_called_once_with(
            test.IsHttpRequest(), vg_snapshot.id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from django.urls import reverse
from django.utils import html

from openstack_dashboard import api
from openstack_dashboard.dashboards.admin.aggregates import constants
from openstack_dashboard.dashboards.admin.aggregates import workflows
from openstack_dashboard.test import helpers as test


class BaseAggregateWorkflowTests(test.BaseAdminViewTests):

    def _get_create_workflow_data(self, aggregate, hosts=None):
        aggregate_info = {"name": aggregate.name,
                          "availability_zone": aggregate.availability_zone}

        if hosts:
            host_field_name = 'add_host_to_aggregate_role_member'
            aggregate_info[host_field_name] = hosts

        return aggregate_info


class CreateAggregateWorkflowTests(BaseAggregateWorkflowTests):

    @test.create_mocks({api.nova: ['service_list']})
    def test_workflow_get(self):

        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        self.mock_service_list.return_value = compute_services

        url = reverse(constants.AGGREGATES_CREATE_URL)
        res = self.client.get(url)
        workflow = res.context['workflow']

        self.assertTemplateUsed(res, constants.AGGREGATES_CREATE_VIEW_TEMPLATE)
        self.assertEqual(workflow.name, workflows.CreateAggregateWorkflow.name)
        self.assertQuerysetEqual(
            workflow.steps,
            ['<SetAggregateInfoStep: set_aggregate_info>',
             '<AddHostsToAggregateStep: add_host_to_aggregate>'])
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(),
            binary='nova-compute')

    @test.create_mocks({
        api.nova: ['service_list',
                   'aggregate_details_list',
                   'aggregate_create']})
    def _test_generic_create_aggregate(self, workflow_data, aggregate,
                                       existing_aggregates=(),
                                       error_count=0,
                                       expected_error_message=None):
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        self.mock_service_list.return_value = compute_services
        self.mock_aggregate_details_list.return_value = existing_aggregates

        if not expected_error_message:
            self.mock_aggregate_create.return_value = aggregate

        url = reverse(constants.AGGREGATES_CREATE_URL)
        res = self.client.post(url, workflow_data)

        if not expected_error_message:
            self.assertNoFormErrors(res)
            self.assertRedirectsNoFollow(
                res, reverse(constants.AGGREGATES_INDEX_URL))
            self.mock_aggregate_create.assert_called_once_with(
                test.IsHttpRequest(),
                name=workflow_data['name'],
                availability_zone=workflow_data['availability_zone'])
        else:
            self.assertFormErrors(res, error_count, expected_error_message)
            self.mock_aggregate_create.assert_not_called()
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(),
            binary='nova-compute')
        self.mock_aggregate_details_list.assert_called_once_with(
            test.IsHttpRequest())

    def test_create_aggregate(self):
        aggregate = self.aggregates.first()
        workflow_data = self._get_create_workflow_data(aggregate)
        self._test_generic_create_aggregate(workflow_data, aggregate)

    def test_create_aggregate_fails_missing_fields(self):
        aggregate = self.aggregates.first()
        workflow_data = self._get_create_workflow_data(aggregate)
        workflow_data['name'] = ''
        workflow_data['availability_zone'] = ''
        self._test_generic_create_aggregate(workflow_data, aggregate, (), 1,
                                            u'This field is required')

    def test_create_aggregate_fails_missing_fields_existing_aggregates(self):
        aggregate = self.aggregates.first()
        existing_aggregates = self.aggregates.list()
        workflow_data = self._get_create_workflow_data(aggregate)
        workflow_data['name'] = ''
        workflow_data['availability_zone'] = ''

        self._test_generic_create_aggregate(workflow_data, aggregate,
                                            existing_aggregates, 1,
                                            u'This field is required')

    def test_create_aggregate_fails_duplicated_name(self):
        aggregate = self.aggregates.first()
        existing_aggregates = self.aggregates.list()
        workflow_data = self._get_create_workflow_data(aggregate)
        expected_error_message = html \
            .escape(u'The name "%s" is already used by another host aggregate.'
                    % aggregate.name)

        self._test_generic_create_aggregate(workflow_data, aggregate,
                                            existing_aggregates, 1,
                                            expected_error_message)

    @test.create_mocks(
        {api.nova: ['service_list',
                    'aggregate_details_list',
                    'aggregate_create',
                    'add_host_to_aggregate']})
    def test_create_aggregate_with_hosts(self):
        aggregate = self.aggregates.first()

        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        compute_hosts = [s.host for s in compute_services]

        self.mock_service_list.return_value = compute_services
        self.mock_aggregate_details_list.return_value = []

        workflow_data = self._get_create_workflow_data(aggregate,
                                                       compute_hosts)
        self.mock_aggregate_create.return_value = aggregate
        self.mock_add_host_to_aggregate.return_value = None

        url = reverse(constants.AGGREGATES_CREATE_URL)
        res = self.client.post(url, workflow_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res,
                                     reverse(constants.AGGREGATES_INDEX_URL))
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(),
            binary='nova-compute')
        self.mock_aggregate_details_list.assert_called_once_with(
            test.IsHttpRequest())
        self.mock_aggregate_create.assert_called_once_with(
            test.IsHttpRequest(),
            name=workflow_data['name'],
            availability_zone=workflow_data['availability_zone'])
        self.assertEqual(self.mock_add_host_to_aggregate.call_count,
                         len(compute_hosts))

        expected_calls = [mock.call(test.IsHttpRequest(), aggregate.id, h)
                          for h in compute_hosts]
        self.mock_add_host_to_aggregate.assert_has_calls(
            expected_calls)

    @test.create_mocks({api.nova: ['service_list']})
    def test_service_list_nova_compute(self):
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        self.mock_service_list.return_value = compute_services

        url = reverse(constants.AGGREGATES_CREATE_URL)
        res = self.client.get(url)
        workflow = res.context['workflow']
        step = workflow.get_step("add_host_to_aggregate")
        field_name = step.get_member_field_name('member')
        self.assertEqual(len(step.action.fields[field_name].choices),
                         len(compute_services))
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(), binary='nova-compute')


class AggregatesViewTests(test.BaseAdminViewTests):

    @test.create_mocks({
        api.keystone: ['tenant_list'],
        api.nova: ['extension_supported']})
    def test_panel_not_available(self):
        self.mock_tenant_list.return_value = self.tenants.list()
        self.mock_extension_supported.return_value = False

        self.patchers['aggregates'].stop()
        res = self.client.get(reverse('horizon:admin:overview:index'))
        self.assertNotIn(b'Host Aggregates', res.content)
        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        self.assertEqual(self.mock_extension_supported.call_count, 3)
        expected_calls = [mock.call(a, test.IsHttpRequest())
                          for a in ['SimpleTenantUsage',
                                    'SimpleTenantUsage',
                                    'Aggregates']]
        self.mock_extension_supported.assert_has_calls(
            expected_calls)

    @test.create_mocks({
        api.nova: ['aggregate_details_list',
                   'availability_zone_list']})
    def test_index(self):
        self.mock_aggregate_details_list.return_value = self.aggregates.list()
        self.mock_availability_zone_list.return_value = \
            self.availability_zones.list()

        res = self.client.get(reverse(constants.AGGREGATES_INDEX_URL))
        self.assertTemplateUsed(res, constants.AGGREGATES_INDEX_VIEW_TEMPLATE)
        self.assertItemsEqual(res.context['host_aggregates_table'].data,
                              self.aggregates.list())
        self.assertItemsEqual(res.context['availability_zones_table'].data,
                              self.availability_zones.list())
        self.mock_aggregate_details_list.assert_called_once_with(
            test.IsHttpRequest())
        self.mock_availability_zone_list.assert_called_once_with(
            test.IsHttpRequest(), detailed=True)

    @test.create_mocks({api.nova: ['aggregate_update', 'aggregate_get']})
    def _test_generic_update_aggregate(self, form_data, aggregate,
                                       error_count=0,
                                       expected_error_message=None):
        self.mock_aggregate_get.return_value = aggregate
        if not expected_error_message:
            az = form_data['availability_zone']
            aggregate_data = {'name': form_data['name'],
                              'availability_zone': az}
            self.mock_aggregate_update.return_value = None

        res = self.client.post(reverse(constants.AGGREGATES_UPDATE_URL,
                               args=[aggregate.id]),
                               form_data)

        self.mock_aggregate_get.assert_called_once_with(
            test.IsHttpRequest(),
            str(aggregate.id))
        if not expected_error_message:
            self.assertNoFormErrors(res)
            self.assertRedirectsNoFollow(
                res, reverse(constants.AGGREGATES_INDEX_URL))
            self.mock_aggregate_update.assert_called_once_with(
                test.IsHttpRequest(),
                str(aggregate.id),
                aggregate_data)
        else:
            self.assertFormErrors(res, error_count, expected_error_message)

    def test_update_aggregate(self):
        aggregate = self.aggregates.first()
        form_data = {'id': aggregate.id,
                     'name': 'my_new_name',
                     'availability_zone': 'my_new_zone'}

        self._test_generic_update_aggregate(form_data, aggregate)

    def test_update_aggregate_fails_missing_fields(self):
        aggregate = self.aggregates.first()
        form_data = {'id': aggregate.id}

        self._test_generic_update_aggregate(form_data, aggregate, 1,
                                            u'This field is required')


class ManageHostsTests(test.BaseAdminViewTests):

    @test.create_mocks({api.nova: ['aggregate_get', 'service_list']})
    def test_manage_hosts(self):
        aggregate = self.aggregates.first()

        self.mock_aggregate_get.return_value = aggregate
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        self.mock_service_list.return_value = compute_services

        res = self.client.get(reverse(constants.AGGREGATES_MANAGE_HOSTS_URL,
                                      args=[aggregate.id]))
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res,
                                constants.AGGREGATES_MANAGE_HOSTS_TEMPLATE)
        self.mock_aggregate_get.assert_called_once_with(
            test.IsHttpRequest(),
            str(aggregate.id))
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(),
            binary='nova-compute')

    @test.create_mocks({
        api.nova: ['aggregate_get',
                   'add_host_to_aggregate',
                   'remove_host_from_aggregate',
                   'service_list']})
    def test_manage_hosts_update_add_remove_not_empty_aggregate(self):
        aggregate = self.aggregates.first()
        aggregate.hosts = ['host1', 'host2']
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        host = compute_services[0].host
        form_data = {'manageaggregatehostsaction_role_member': [host]}

        self.mock_remove_host_from_aggregate.return_value = None
        self.mock_aggregate_get.return_value = aggregate
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        self.mock_service_list.return_value = compute_services
        self.mock_add_host_to_aggregate.return_value = None

        res = self.client.post(reverse(constants.AGGREGATES_MANAGE_HOSTS_URL,
                                       args=[aggregate.id]),
                               form_data)
        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res,
                                     reverse(constants.AGGREGATES_INDEX_URL))
        self.assertEqual(self.mock_aggregate_get.call_count, 2)
        self.mock_aggregate_get.assert_has_calls(
            [mock.call(test.IsHttpRequest(), str(aggregate.id))] * 2)
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(), binary='nova-compute')
        self.mock_add_host_to_aggregate.assert_called_once_with(
            test.IsHttpRequest(), str(aggregate.id), host)
        self.assertEqual(self.mock_remove_host_from_aggregate.call_count, 2)
        expected_calls = [mock.call(test.IsHttpRequest(), str(aggregate.id), h)
                          for h in ('host1', 'host2')]
        self.mock_remove_host_from_aggregate.assert_has_calls(
            expected_calls,
            any_order=True)

    @test.create_mocks({
        api.nova: ['aggregate_get',
                   'add_host_to_aggregate',
                   'remove_host_from_aggregate',
                   'service_list']})
    def test_manage_hosts_update_add_not_empty_aggregate_should_fail(self):
        aggregate = self.aggregates.first()
        aggregate.hosts = ['devstack001']
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        host1 = compute_services[0].host
        host3 = compute_services[2].host
        form_data = {'manageaggregatehostsaction_role_member': [host1, host3]}

        self.mock_aggregate_get.return_value = aggregate
        self.mock_service_list.return_value = compute_services
        self.mock_add_host_to_aggregate.side_effect = self.exceptions.nova

        res = self.client.post(reverse(constants.AGGREGATES_MANAGE_HOSTS_URL,
                                       args=[aggregate.id]),
                               form_data)
        self.assertNoFormErrors(res)
        self.assertMessageCount(error=2)
        self.assertRedirectsNoFollow(res,
                                     reverse(constants.AGGREGATES_INDEX_URL))
        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_aggregate_get,
            2,
            mock.call(test.IsHttpRequest(), str(aggregate.id)))
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(), binary='nova-compute')
        self.mock_add_host_to_aggregate.assert_called_once_with(
            test.IsHttpRequest(), str(aggregate.id), host3)

    @test.create_mocks({
        api.nova: ['aggregate_get',
                   'add_host_to_aggregate',
                   'remove_host_from_aggregate',
                   'service_list']})
    def test_manage_hosts_update_clean_not_empty_aggregate_should_fail(self):
        aggregate = self.aggregates.first()
        aggregate.hosts = ['host2']
        form_data = {'manageaggregatehostsaction_role_member':
                     []}

        self.mock_remove_host_from_aggregate.side_effect = self.exceptions.nova
        self.mock_aggregate_get.return_value = aggregate
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        self.mock_service_list.return_value = compute_services

        res = self.client.post(reverse(constants.AGGREGATES_MANAGE_HOSTS_URL,
                                       args=[aggregate.id]),
                               form_data)

        self.assertNoFormErrors(res)
        self.assertMessageCount(error=2)
        self.assertRedirectsNoFollow(res,
                                     reverse(constants.AGGREGATES_INDEX_URL))
        self.mock_remove_host_from_aggregate.assert_called_once_with(
            test.IsHttpRequest(),
            str(aggregate.id),
            'host2')
        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_aggregate_get,
            2,
            mock.call(test.IsHttpRequest(), str(aggregate.id)))
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(),
            binary='nova-compute')

    @test.create_mocks({
        api.nova: ['aggregate_get',
                   'add_host_to_aggregate',
                   'remove_host_from_aggregate',
                   'service_list']})
    def _test_manage_hosts_update(self,
                                  host,
                                  aggregate,
                                  form_data,
                                  add_aggregate=False,
                                  clean_aggregates=False):
        if clean_aggregates:
            self.mock_remove_host_from_aggregate.return_value = None
        self.mock_aggregate_get.return_value = aggregate
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        self.mock_service_list.return_value = compute_services
        if add_aggregate:
            self.mock_add_host_to_aggregate.return_value = None

        res = self.client.post(reverse(constants.AGGREGATES_MANAGE_HOSTS_URL,
                                       args=[aggregate.id]),
                               form_data)
        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res,
                                     reverse(constants.AGGREGATES_INDEX_URL))
        if clean_aggregates:
            self.assertEqual(self.mock_remove_host_from_aggregate.call_count,
                             3)
            expected_calls = [mock.call(test.IsHttpRequest(),
                                        str(aggregate.id), h)
                              for h in ('host1', 'host2', 'host3')]
            self.mock_remove_host_from_aggregate.assert_has_calls(
                expected_calls,
                any_order=True)
        else:
            self.mock_remove_host_from_aggregate.assert_not_called()

        if add_aggregate:
            self.mock_add_host_to_aggregate.assert_called_once_with(
                test.IsHttpRequest(),
                str(aggregate.id),
                host)
        else:
            self.mock_add_host_to_aggregate.assert_not_called()

        self.assert_mock_multiple_calls_with_same_arguments(
            self.mock_aggregate_get,
            2,
            mock.call(test.IsHttpRequest(), str(aggregate.id)))
        self.mock_service_list.assert_called_once_with(
            test.IsHttpRequest(),
            binary='nova-compute')

    def test_manage_hosts_update_nothing_not_empty_aggregate(self):
        aggregate = self.aggregates.first()
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        host = compute_services[0].host
        aggregate.hosts = [host]
        form_data = {'manageaggregatehostsaction_role_member': [host]}
        self._test_manage_hosts_update(host,
                                       aggregate,
                                       form_data,
                                       add_aggregate=False)

    def test_manage_hosts_update_nothing_empty_aggregate(self):
        aggregate = self.aggregates.first()
        aggregate.hosts = []
        form_data = {'manageaggregatehostsaction_role_member':
                     []}
        self._test_manage_hosts_update(None,
                                       aggregate,
                                       form_data,
                                       add_aggregate=False)

    def test_manage_hosts_update_add_empty_aggregate(self):
        aggregate = self.aggregates.first()
        aggregate.hosts = []
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        host = compute_services[0].host
        form_data = {'manageaggregatehostsaction_role_member': [host]}
        self._test_manage_hosts_update(host,
                                       aggregate,
                                       form_data,
                                       add_aggregate=True)

    def test_manage_hosts_update_add_not_empty_aggregate(self):
        aggregate = self.aggregates.first()
        aggregate.hosts = ['devstack001']
        compute_services = [s for s in self.services.list()
                            if s.binary == 'nova-compute']
        host1 = compute_services[0].host
        host3 = compute_services[2].host
        form_data = {'manageaggregatehostsaction_role_member': [host1, host3]}
        self._test_manage_hosts_update(host3,
                                       aggregate,
                                       form_data,
                                       add_aggregate=True)

    def test_manage_hosts_update_clean_not_empty_aggregate(self):
        aggregate = self.aggregates.first()
        aggregate.hosts = ['host1', 'host2', 'host3']
        form_data = {'manageaggregatehostsaction_role_member':
                     []}
        self._test_manage_hosts_update(None,
                                       aggregate,
                                       form_data,
                                       add_aggregate=False,
                                       clean_aggregates=True)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.views import generic

from openstack_dashboard.api.rest import urls
from openstack_dashboard.api.rest import utils as rest_utils
from openstack_dashboard import policy


@urls.register
class Policy(generic.View):
    '''API for interacting with the policy engine.'''

    url_regex = r'policy/$'

    @rest_utils.ajax(data_required=True)
    def post(self, request):
        '''Check policy rules.

        Check the group of policy rules supplied in the POST
        application/json object. The policy target, if specified will also be
        passed in to the policy check method as well.

        The action returns an object with one key: "allowed" and the value
        is the result of the policy check, True or False.
        '''

        rules = []
        try:
            rules_in = request.DATA['rules']
            rules = tuple([tuple(rule) for rule in rules_in])
        except Exception:
            raise rest_utils.AjaxError(400, 'unexpected parameter format')

        policy_target = request.DATA.get('target') or {}

        result = policy.check(rules, request, policy_target)

        return {"allowed": result}



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from keystoneauth1.identity import v2 as v2_auth
from keystoneauth1.identity import v3 as v3_auth

from openstack_auth.plugin import base
from openstack_auth import utils

LOG = logging.getLogger(__name__)

__all__ = ['PasswordPlugin']


class PasswordPlugin(base.BasePlugin):
    """Authenticate against keystone given a username and password.

    This is the default login mechanism. Given a username and password inputted
    from a login form returns a v2 or v3 keystone Password plugin for
    authentication.
    """

    def get_plugin(self, auth_url=None, username=None, password=None,
                   user_domain_name=None, **kwargs):
        if not all((auth_url, username, password)):
            return None

        LOG.debug('Attempting to authenticate for %s', username)

        if utils.get_keystone_version() >= 3:
            return v3_auth.Password(auth_url=auth_url,
                                    username=username,
                                    password=password,
                                    user_domain_name=user_domain_name,
                                    unscoped=True)

        else:
            return v2_auth.Password(auth_url=auth_url,
                                    username=username,
                                    password=password)



# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

from django.conf import settings
from django import http
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import views


ADD_TO_FIELD_HEADER = "HTTP_X_HORIZON_ADD_TO_FIELD"


class ModalBackdropMixin(object):
    """Mixin class to allow ModalFormView and WorkflowView together.

    This mixin class is to be used for together with ModalFormView and
    WorkflowView classes to augment them with modal_backdrop context data.

    .. attribute: modal_backdrop (optional)

        The appearance and behavior of backdrop under the modal element.
        Possible options are:
        * 'true' - show backdrop element outside the modal, close the modal
        after clicking on backdrop (the default one);
        * 'false' - do not show backdrop element, do not close the modal after
        clicking outside of it;
        * 'static' - show backdrop element outside the modal, do not close
        the modal after clicking on backdrop.
    """
    modal_backdrop = 'static'

    def __init__(self, *args, **kwargs):
        super(ModalBackdropMixin, self).__init__(*args, **kwargs)
        config = getattr(settings, 'HORIZON_CONFIG', {})
        if 'modal_backdrop' in config:
            self.modal_backdrop = config['modal_backdrop']

    def get_context_data(self, **kwargs):
        context = super(ModalBackdropMixin, self).get_context_data(**kwargs)
        context['modal_backdrop'] = self.modal_backdrop
        return context


class ModalFormMixin(ModalBackdropMixin):
    def get_template_names(self):
        if self.request.is_ajax():
            if not hasattr(self, "ajax_template_name"):
                # Transform standard template name to ajax name (leading "_")
                bits = list(os.path.split(self.template_name))
                bits[1] = "".join(("_", bits[1]))
                self.ajax_template_name = os.path.join(*bits)
            template = self.ajax_template_name
        else:
            template = self.template_name
        return template

    def get_context_data(self, **kwargs):
        context = super(ModalFormMixin, self).get_context_data(**kwargs)
        if self.request.is_ajax():
            context['hide'] = True
        if ADD_TO_FIELD_HEADER in self.request.META:
            context['add_to_field'] = self.request.META[ADD_TO_FIELD_HEADER]
        return context


class ModalFormView(ModalFormMixin, views.HorizonFormView):
    """The main view class for all views which handle forms in Horizon.

    All view which handles forms in Horiozn should inherit this class.
    It takes care of all details with processing
    :class:`~horizon.forms.base.SelfHandlingForm` classes, and modal concerns
    when the associated template inherits from
    `horizon/common/_modal_form.html`.

    Subclasses must define a ``form_class`` and ``template_name`` attribute
    at minimum.

    See Django's documentation on the `FormView <https://docs.djangoproject.com
    /en/dev/ref/class-based-views/generic-editing/#formview>`_ class for
    more details.

    .. attribute: modal_id (recommended)

        The HTML element id of this modal.

    .. attribute: modal_header (recommended)

        The title of this modal.

    .. attribute: form_id (recommended)

        The HTML element id of the form in this modal.

    .. attribute: submit_url (required)

        The url for a submit action.

    .. attribute: submit_label (optional)

        The label for the submit button. This label defaults to ``Submit``.
        This button should only be visible if the action_url is defined.
        Clicking on this button will post to the action_url.

    .. attribute: cancel_label (optional)

        The label for the cancel button. This label defaults to ``Cancel``.
        Clicking on this button will redirect user to the cancel_url.

    .. attribute: cancel_url (optional)

        The url for a cancel action. This url defaults to the success_url
        if omitted. Note that the cancel_url redirect is nullified when
        shown in a modal dialog.
    """

    modal_id = None
    modal_header = ""
    form_id = None
    submit_url = None
    submit_label = _("Submit")
    cancel_label = _("Cancel")
    cancel_url = None

    def get_context_data(self, **kwargs):
        context = super(ModalFormView, self).get_context_data(**kwargs)
        context['modal_id'] = self.modal_id
        context['modal_header'] = self.modal_header
        context['form_id'] = self.form_id
        context['submit_url'] = self.submit_url
        context['submit_label'] = self.submit_label
        context['cancel_label'] = self.cancel_label
        context['cancel_url'] = self.get_cancel_url()
        return context

    def get_cancel_url(self):
        return self.cancel_url or self.success_url

    def get_object_id(self, obj):
        """Returns the ID of the created object.

        For  dynamic insertion of resources created in modals,
        this method returns the id of the created object.
        Defaults to returning the ``id`` attribute.
        """
        return obj.id

    def get_object_display(self, obj):
        """Returns the display name of the created object.

        For dynamic insertion of resources created in modals,
        this method returns the display name of the created object.
        Defaults to returning the ``name`` attribute.
        """
        return obj.name

    def get_form(self, form_class=None):
        """Returns an instance of the form to be used in this view."""
        if form_class is None:
            form_class = self.get_form_class()
        return form_class(self.request, **self.get_form_kwargs())

    def form_invalid(self, form):
        context = self.get_context_data()
        context['form'] = form
        return self.render_to_response(context)

    def form_valid(self, form):
        try:
            handled = form.handle(self.request, form.cleaned_data)
        except Exception:
            handled = None
            exceptions.handle(self.request)

        if handled:
            if ADD_TO_FIELD_HEADER in self.request.META:
                field_id = self.request.META[ADD_TO_FIELD_HEADER]
                data = [self.get_object_id(handled),
                        self.get_object_display(handled)]
                response = http.HttpResponse(
                    json.dumps(data), content_type="text/plain")
                response["X-Horizon-Add-To-Field"] = field_id
            elif isinstance(handled, http.HttpResponse):
                return handled
            else:
                try:
                    success_url = self.get_success_url_from_handled(handled)
                except AttributeError:
                    success_url = self.get_success_url()

                response = http.HttpResponseRedirect(success_url)
                if hasattr(handled, 'to_dict'):
                    obj_dict = handled.to_dict()
                    if 'upload_url' in obj_dict:
                        response['X-File-Upload-URL'] = obj_dict['upload_url']
                        response['X-Auth-Token'] = obj_dict['token_id']
                # TODO(gabriel): This is not a long-term solution to how
                # AJAX should be handled, but it's an expedient solution
                # until the blueprint for AJAX handling is architected
                # and implemented.
                response['X-Horizon-Location'] = success_url
            return response
        else:
            # If handled didn't return, we can assume something went
            # wrong, and we should send back the form as-is.
            return self.form_invalid(form)



# Copyright 2016 Letv Cloud Computing
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse

from openstack_dashboard import api
from openstack_dashboard.test import helpers as test

INDEX_URL = reverse('horizon:admin:floating_ips:index')
INDEX_TEMPLATE = 'horizon/common/_data_table_view.html'


class AdminFloatingIpViewTest(test.BaseAdminViewTests):

    @test.create_mocks({
        api.nova: ['server_list'],
        api.keystone: ['tenant_list'],
        api.neutron: ['network_list',
                      'is_extension_supported',
                      'tenant_floating_ip_list']})
    def test_index(self):
        # Use neutron test data
        fips = self.floating_ips.list()
        servers = self.servers.list()
        tenants = self.tenants.list()
        self.mock_tenant_floating_ip_list.return_value = fips
        self.mock_server_list.return_value = [servers, False]
        self.mock_tenant_list.return_value = [tenants, False]
        self.mock_network_list.return_value = self.networks.list()
        self.mock_is_extension_supported.return_value = True

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.assertIn('floating_ips_table', res.context)
        floating_ips_table = res.context['floating_ips_table']
        floating_ips = floating_ips_table.data
        self.assertEqual(len(floating_ips), 2)

        row_actions = floating_ips_table.get_row_actions(floating_ips[0])
        self.assertEqual(len(row_actions), 1)
        row_actions = floating_ips_table.get_row_actions(floating_ips[1])
        self.assertEqual(len(row_actions), 2)
        self.mock_tenant_floating_ip_list.assert_called_once_with(
            test.IsHttpRequest(),
            all_tenants=True)
        self.mock_server_list.assert_called_once_with(
            test.IsHttpRequest(),
            detailed=False,
            search_opts={'all_tenants': True})
        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        params = {"router:external": True}
        self.mock_network_list.assert_called_once_with(
            test.IsHttpRequest(), **params)
        self.mock_is_extension_supported.assert_called_once_with(
            test.IsHttpRequest(), 'dns-integration')

    @test.create_mocks({
        api.neutron: ['network_get',
                      'is_extension_supported',
                      'tenant_floating_ip_get']})
    def test_floating_ip_detail_get(self):
        fip = self.floating_ips.first()
        network = self.networks.first()
        self.mock_tenant_floating_ip_get.return_value = fip
        self.mock_network_get.return_value = network
        self.mock_is_extension_supported.return_value = True

        res = self.client.get(reverse('horizon:admin:floating_ips:detail',
                                      args=[fip.id]))
        self.assertTemplateUsed(res,
                                'admin/floating_ips/detail.html')
        self.assertEqual(res.context['floating_ip'].ip, fip.ip)
        self.mock_tenant_floating_ip_get.assert_called_once_with(
            test.IsHttpRequest(), fip.id)
        self.mock_network_get.assert_called_once_with(
            test.IsHttpRequest(), fip.pool)
        self.mock_is_extension_supported.assert_called_once_with(
            test.IsHttpRequest(), 'dns-integration')

    @test.create_mocks({api.neutron: ['tenant_floating_ip_get']})
    def test_floating_ip_detail_exception(self):
        fip = self.floating_ips.first()
        # Only supported by neutron, so raise a neutron exception
        self.mock_tenant_floating_ip_get.side_effect = self.exceptions.neutron

        res = self.client.get(reverse('horizon:admin:floating_ips:detail',
                                      args=[fip.id]))

        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.mock_tenant_floating_ip_get.assert_called_once_with(
            test.IsHttpRequest(), fip.id)

    @test.create_mocks({api.neutron: ['tenant_floating_ip_list',
                                      'is_extension_supported']})
    def test_index_no_floating_ips(self):
        self.mock_tenant_floating_ip_list.return_value = []
        self.mock_is_extension_supported.return_value = True

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.mock_tenant_floating_ip_list.assert_called_once_with(
            test.IsHttpRequest(), all_tenants=True)
        self.mock_is_extension_supported.assert_called_once_with(
            test.IsHttpRequest(), 'dns-integration')

    @test.create_mocks({api.neutron: ['tenant_floating_ip_list',
                                      'is_extension_supported']})
    def test_index_error(self):
        self.mock_tenant_floating_ip_list.side_effect = self.exceptions.neutron
        self.mock_is_extension_supported.return_value = True

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.mock_tenant_floating_ip_list.assert_called_once_with(
            test.IsHttpRequest(), all_tenants=True)
        self.mock_is_extension_supported.assert_called_once_with(
            test.IsHttpRequest(), 'dns-integration')

    @test.create_mocks({
        api.neutron: ['network_list'],
        api.keystone: ['tenant_list']})
    def test_admin_allocate_get(self):
        pool = self.networks.first()
        tenants = self.tenants.list()

        self.mock_tenant_list.return_value = [tenants, False]
        self.mock_network_list.return_value = [pool]

        url = reverse('horizon:admin:floating_ips:allocate')
        res = self.client.get(url)
        self.assertTemplateUsed(res, 'admin/floating_ips/allocate.html')
        allocate_form = res.context['form']

        pool_choices = allocate_form.fields['pool'].choices
        self.assertEqual(len(pool_choices), 1)
        tenant_choices = allocate_form.fields['tenant'].choices
        self.assertEqual(len(tenant_choices), 3)
        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        search_opts = {'router:external': True}
        self.mock_network_list.assert_called_once_with(
            test.IsHttpRequest(), **search_opts)

    @test.create_mocks({
        api.neutron: ['network_list'],
        api.keystone: ['tenant_list']})
    def test_admin_allocate_post_invalid_ip_version(self):
        tenant = self.tenants.first()
        pool = self.networks.first()
        tenants = self.tenants.list()

        self.mock_tenant_list.return_value = [tenants, False]
        self.mock_network_list.return_value = [pool]

        form_data = {'pool': pool.id,
                     'tenant': tenant.id,
                     'floating_ip_address': 'fc00::1'}
        url = reverse('horizon:admin:floating_ips:allocate')
        res = self.client.post(url, form_data)
        self.assertContains(res, "Invalid version for IP address")
        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        search_opts = {'router:external': True}
        self.mock_network_list.assert_called_once_with(
            test.IsHttpRequest(), **search_opts)

    @test.create_mocks({
        api.neutron: ['tenant_floating_ip_allocate',
                      'network_list',
                      'subnet_get'],
        api.keystone: ['tenant_list']})
    def test_admin_allocate_post(self):
        tenant = self.tenants.first()
        floating_ip = self.floating_ips.first()
        subnet = self.subnets.first()
        pool = self.networks.first()
        tenants = self.tenants.list()

        self.mock_subnet_get.return_value = subnet
        self.mock_tenant_list.return_value = [tenants, False]
        self.mock_network_list.return_value = [pool]
        self.mock_tenant_floating_ip_allocate.return_value = floating_ip

        form_data = {'pool': subnet.id,
                     'tenant': tenant.id}
        url = reverse('horizon:admin:floating_ips:allocate')
        res = self.client.post(url, form_data)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        self.mock_subnet_get.assert_called_once_with(
            test.IsHttpRequest(), subnet.id)
        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        search_opts = {'router:external': True}
        self.mock_network_list.assert_called_once_with(
            test.IsHttpRequest(), **search_opts)
        self.mock_tenant_floating_ip_allocate.assert_called_once_with(
            test.IsHttpRequest(),
            pool=pool.id,
            subnet_id=subnet.id,
            tenant_id=tenant.id)

    @test.create_mocks({
        api.neutron: ['tenant_floating_ip_list',
                      'floating_ip_disassociate',
                      'is_extension_supported',
                      'network_list'],
        api.nova: ['server_list'],
        api.keystone: ['tenant_list']})
    def test_admin_disassociate_floatingip(self):
        # Use neutron test data
        fips = self.floating_ips.list()
        floating_ip = self.floating_ips.list()[1]
        servers = self.servers.list()
        tenants = self.tenants.list()
        self.mock_tenant_floating_ip_list.return_value = fips
        self.mock_server_list.return_value = [servers, False]
        self.mock_tenant_list.return_value = [tenants, False]
        self.mock_network_list.return_value = self.networks.list()
        self.mock_floating_ip_disassociate.return_value = None
        self.mock_is_extension_supported.return_value = True

        form_data = {
            "action":
            "floating_ips__disassociate__%s" % floating_ip.id}
        res = self.client.post(INDEX_URL, form_data)

        self.assertNoFormErrors(res)
        self.mock_tenant_floating_ip_list.assert_called_once_with(
            test.IsHttpRequest(), all_tenants=True)
        self.mock_server_list.assert_called_once_with(
            test.IsHttpRequest(),
            detailed=False,
            search_opts={'all_tenants': True})
        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        params = {"router:external": True}
        self.mock_network_list.assert_called_once_with(
            test.IsHttpRequest(), **params)
        self.mock_floating_ip_disassociate.assert_called_once_with(
            test.IsHttpRequest(), floating_ip.id)
        self.mock_is_extension_supported.assert_called_once_with(
            test.IsHttpRequest(), 'dns-integration')

    @test.create_mocks({
        api.neutron: ['tenant_floating_ip_list',
                      'is_extension_supported',
                      'network_list'],
        api.nova: ['server_list'],
        api.keystone: ['tenant_list']})
    def test_admin_delete_floatingip(self):
        # Use neutron test data
        fips = self.floating_ips.list()
        floating_ip = self.floating_ips.list()[1]
        servers = self.servers.list()
        tenants = self.tenants.list()
        self.mock_tenant_floating_ip_list.return_value = fips
        self.mock_server_list.return_value = [servers, False]
        self.mock_tenant_list.return_value = [tenants, False]
        self.mock_network_list.return_value = self.networks.list()
        self.mock_is_extension_supported.return_value = True

        form_data = {
            "action":
            "floating_ips__delete__%s" % floating_ip.id}
        res = self.client.post(INDEX_URL, form_data)

        self.assertNoFormErrors(res)
        self.mock_tenant_floating_ip_list.assert_called_once_with(
            test.IsHttpRequest(), all_tenants=True)
        self.mock_server_list.assert_called_once_with(
            test.IsHttpRequest(),
            detailed=False,
            search_opts={'all_tenants': True})
        self.mock_tenant_list.assert_called_once_with(test.IsHttpRequest())
        params = {"router:external": True}
        self.mock_network_list.assert_called_once_with(
            test.IsHttpRequest(), **params)
        self.mock_is_extension_supported.assert_called_once_with(
            test.IsHttpRequest(), 'dns-integration')

    @test.create_mocks({
        api.neutron: ['tenant_floating_ip_list',
                      'is_extension_supported',
                      'network_list'],
        api.nova: ['server_list'],
        api.keystone: ['tenant_list']})
    def test_floating_ip_table_actions(self):
        # Use neutron test data
        fips = self.floating_ips.list()
        servers = self.servers.list()
        tenants = self.tenants.list()
        self.mock_tenant_floating_ip_list.return_value = fips
        self.mock_server_list.return_value = [servers, False]
        self.mock_tenant_list.return_value = [tenants, False]
        self.mock_network_list.return_value = self.networks.list()
        self.mock_is_extension_supported.return_value = True

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        self.assertIn('floating_ips_table', res.context)
        floating_ips_table = res.context['floating_ips_table']
        floating_ips = floating_ips_table.data
        self.assertEqual(len(floating_ips), 2)
        # table actions
        self.assertContains(res, 'id="floating_ips__action_allocate"')
        self.assertContains(res, 'id="floating_ips__action_release"')
        # row actions
        self.assertContains(res, 'floating_ips__release__%s' % fips[0].id)
        self.assertContains(res, 'floating_ips__release__%s' % fips[1].id)
        self.assertContains(res, 'floating_ips__disassociate__%s' % fips[1].id)
        self.mock_tenant_floating_ip_list.assert_called_once_with(
            test.IsHttpRequest(), all_tenants=True)
        self.mock_server_list.assert_called_once_with(
            test.IsHttpRequest(),
            detailed=False,
            search_opts={'all_tenants': True})
        params = {"router:external": True}
        self.mock_network_list.assert_called_once_with(
            test.IsHttpRequest(), **params)
        self.mock_is_extension_supported.assert_called_once_with(
            test.IsHttpRequest(), 'dns-integration')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from selenium.webdriver.common import by

from openstack_dashboard.test.integration_tests.pages import basepage
from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import menus
from openstack_dashboard.test.integration_tests.regions import tables


class FlavorsTable(tables.TableRegion):
    name = "flavors"

    CREATE_FLAVOR_FORM_FIELDS = (("name", "flavor_id", "vcpus", "memory_mb",
                                  "disk_gb", "eph_gb",
                                  "swap_mb",
                                  "rxtx_factor"),
                                 {"members": menus.MembershipMenuRegion})

    UPDATE_FLAVOR_FORM_FIELDS = (("name", "vcpus", "memory_mb",
                                  "disk_gb", "eph_gb", "swap_mb",
                                  "rxtx_factor"),
                                 {"members": menus.MembershipMenuRegion})

    @tables.bind_table_action('create')
    def create_flavor(self, create_button):
        create_button.click()
        return forms.TabbedFormRegion(
            self.driver,
            self.conf,
            field_mappings=self.CREATE_FLAVOR_FORM_FIELDS
        )

    @tables.bind_row_action('update')
    def update_flavor_info(self, edit_button, row):
        edit_button.click()
        return forms.TabbedFormRegion(
            self.driver,
            self.conf,
            field_mappings=self.UPDATE_FLAVOR_FORM_FIELDS
        )

    @tables.bind_row_action('projects')
    def update_flavor_access(self, update_button, row):
        update_button.click()
        return forms.TabbedFormRegion(
            self.driver,
            self.conf,
            field_mappings=self.UPDATE_FLAVOR_FORM_FIELDS,
            default_tab=1
        )

    @tables.bind_row_action('delete')
    def delete_by_row(self, delete_button, row):
        delete_button.click()
        return forms.BaseFormRegion(self.driver, self.conf)


class FlavorsPage(basepage.BaseNavigationPage):
    DEFAULT_ID = "auto"
    FLAVORS_TABLE_NAME_COLUMN = 'Flavor Name'
    FLAVORS_TABLE_VCPUS_COLUMN = 'VCPUs'
    FLAVORS_TABLE_RAM_COLUMN = 'RAM'
    FLAVORS_TABLE_DISK_COLUMN = 'Root Disk'
    FLAVORS_TABLE_PUBLIC_COLUMN = 'Public'

    def __init__(self, driver, conf):
        super(FlavorsPage, self).__init__(driver, conf)
        self._page_title = "Flavors"

    @property
    def flavors_table(self):
        return FlavorsTable(self.driver, self.conf)

    def _get_flavor_row(self, name):
        return self.flavors_table.get_row(self.FLAVORS_TABLE_NAME_COLUMN, name)

    def create_flavor(self, name, id_=DEFAULT_ID, vcpus=None, ram=None,
                      root_disk=None, ephemeral_disk=None, swap_disk=None):
        create_flavor_form = self.flavors_table.create_flavor()
        create_flavor_form.name.text = name
        if id_ is not None:
            create_flavor_form.flavor_id.text = id_
        create_flavor_form.vcpus.value = vcpus
        create_flavor_form.memory_mb.value = ram
        create_flavor_form.disk_gb.value = root_disk
        create_flavor_form.eph_gb.value = ephemeral_disk
        create_flavor_form.swap_mb.value = swap_disk
        create_flavor_form.submit()

    def is_flavor_present(self, name):
        return bool(self._get_flavor_row(name))

    def update_flavor_info(self, name, add_up):
        row = self._get_flavor_row(name)
        update_flavor_form = self.flavors_table.update_flavor_info(row)

        update_flavor_form.name.text = "edited-" + name
        update_flavor_form.vcpus.value = \
            int(update_flavor_form.vcpus.value) + add_up
        update_flavor_form.memory_mb.value =\
            int(update_flavor_form.memory_mb.value) + add_up
        update_flavor_form.disk_gb.value =\
            int(update_flavor_form.disk_gb.value) + add_up

        update_flavor_form.submit()

    def update_flavor_access(self, name, project_name, allocate=True):
        row = self._get_flavor_row(name)
        update_flavor_form = self.flavors_table.update_flavor_access(row)

        if allocate:
            update_flavor_form.members.allocate_member(project_name)
        else:
            update_flavor_form.members.deallocate_member(project_name)

        update_flavor_form.submit()

    def delete_flavor_by_row(self, name):
        row = self._get_flavor_row(name)
        delete_form = self.flavors_table.delete_by_row(row)
        delete_form.submit()

    def get_flavor_vcpus(self, name):
        row = self._get_flavor_row(name)
        return row.cells[self.FLAVORS_TABLE_VCPUS_COLUMN].text

    def get_flavor_ram(self, name):
        row = self._get_flavor_row(name)
        return row.cells[self.FLAVORS_TABLE_RAM_COLUMN].text

    def get_flavor_disk(self, name):
        row = self._get_flavor_row(name)
        return row.cells[self.FLAVORS_TABLE_DISK_COLUMN].text

    def is_flavor_public(self, name):
        row = self._get_flavor_row(name)
        return row.cells[self.FLAVORS_TABLE_PUBLIC_COLUMN].text == "Yes"


class FlavorsPageNG(FlavorsPage):
    _resource_page_header_locator = (by.By.CSS_SELECTOR,
                                     '.page-header > h1')

    @property
    def header(self):
        return self._get_element(*self._resource_page_header_locator)



# Copyright 2017 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from horizon import tabs

from openstack_dashboard.dashboards.admin.instances import tables
from openstack_dashboard.dashboards.project.instances import tabs \
    as project_tabs


class AuditTab(project_tabs.AuditTab):
    table_classes = (tables.AdminAuditTable,)


class AdminInstanceDetailTabs(tabs.DetailTabsGroup):
    slug = "instance_details"
    tabs = (project_tabs.OverviewTab, project_tabs.LogTab,
            project_tabs.ConsoleTab, AuditTab)
    sticky = True



# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf import settings
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from horizon.browsers.views import AngularIndexView
from openstack_dashboard.dashboards.project.images.images import views


if settings.ANGULAR_FEATURES['images_panel']:
    title = _("Images")
    urlpatterns = [
        url(r'^(?P<image_id>[^/]+)/$', AngularIndexView.as_view(title=title),
            name='detail'),
    ]
else:
    urlpatterns = [
        url(r'^create/$', views.CreateView.as_view(), name='create'),
        url(r'^(?P<image_id>[^/]+)/update/$',
            views.UpdateView.as_view(), name='update'),
        url(r'^(?P<image_id>[^/]+)/$', views.DetailView.as_view(),
            name='detail'),
    ]
